#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import dbus, rpyc, time, traceback, gobject
from net.aircable.utils import getLogger
logger = getLogger(__name__)
from net.aircable.openproximity.signals.scanner import *
from net.aircable.wrappers import Adapter
from rpyc.lib.compat import ByValWrapper
from django.utils.encoding import smart_str

import net.aircable.const as const
import net.aircable.utils as utils
import net.aircable.openproximity.signals as signals

remotescanner_url = "net.aircable.RemoteScanner"

class ScanAdapter(Adapter):
    '''
    Wrapper arround a BlueZ scanner
    '''
    priority = 0
    found = None
    
    def __init__(self, priority, *args, **kwargs):
        Adapter.__init__(self, *args, **kwargs)
        
        if priority is None or priority < 0:
            priority=0
            
        self.priority=int(priority)
        logger.debug("Initializated ScannerDongle: %s" % priority)
    
    def __str__(self):
        return '%s, %s' % (Adapter.__str__(self), self.priority)
    
    def scan(self):
        '''
        Call this function to start a scan process on this dongle
        '''
        self.found = dict()
        self.dbus_interface.StartDiscovery()
        
    def endScan(self):
        '''
        Force ongoing inquiry to stop
        '''
        self.dbus_interface.StopDiscovery()

class RemoteScanAdapter(ScanAdapter):
    '''
    Wrapper arround a scanner to which we connect over bluetooth.
    '''
    priority = 0
    sending = False
    
    def __init__(self, 
                priority, 
                local=None, 
                address=None, 
                bus=None, 
                scanner_manager=None):
        if priority is None or priority < 0:
            priority=0
            
        self.priority=int(priority)
        self.local = local
        self.bt_address = address
        self.bus = bus
        self.scanner_manager = scanner_manager
        
        manager = self.bus.get_object(
            remotescanner_url,
            "/net/aircable/RemoteScanner/Manager"
        )
        self.dbus_path = manager.Connect(dbus_interface=remotescanner_url)
        
        remote_object = self.bus.get_object(remotescanner_url, self.dbus_path)
        
        self.iface = dbus.Interface(remote_object, remotescanner_url)
        
        self.bus.add_signal_receiver(
            self.connected, # callback
            signal_name='ScannerConnected', # signal name
            dbus_interface=remotescanner_url, # interface name
        )

        self.bus.add_signal_receiver(
            self.disconnected, # callback
            signal_name='ScannerDisconnected', # signal name
            dbus_interface=remotescanner_url, # interface name
        )

        logger.debug("Initializated RemoteScannerDongle: %s" % priority)
    
    def connected(self, local, remote):
        '''
        Callback that lets us know when we got connected to the remote scanner
        '''
        if self.bt_address.lower() == remote.lower():
            logger.info("scanner connected %s" % remote)
            self.scanner_manager.property_changed(
                'Discovering',
                0,
                self.dbus_path
            )

    def disconnected(self, local, remote):
        '''
        Callback that lets us know when we got disconnected to the remote 
        scanner
        '''
        if self.bt_address.lower() == remote.lower():
            logger.info("scanner disconnected %s" % remote)
            self.scanner_manager.property_changed(
                'Discovering',
                0,
                self.dbus_path
            )

    
    def __str__(self):
        return 'RemoteScanner %s %s %s' % (
            self.priority, self.local, self.bt_address)
    
    def scan(self):
        '''
        Start a scan in the remote scanner
        '''
        if self.sending:
            return
        
        if not self.iface.isConnected():
            self.iface.Connect(self.local, self.bt_address)
            return
    
        self.sending = True
        self.found = dict()
        self.iface.StartScan(1)
        logger.debug("Scan started")

    def endScan(self):
        '''
        Stop the pending inquiry (has no effect on the RemoteScanner).
        '''
        self.sending = False

class ScanManager:
    '''
    A scanner manager is responsible of handling all the connected scanners, 
    handling all the logic for this and communicating with the server.
    '''
    
    __dongles = dict()
    # dict that holds our in use dongles (ScanAdapter instances)
    
    bus = None
    # a connection to dbus
    
    manager = None
    # a connection to the BlueZ manager
    
    __sequence = list()
    # a holder for our scan sequence (sequence mode)
    
    __index = None
    # index in the scan sequence (sequence mode)
    
    scanners = dict()
    # dict that holds our known dongles (strings)
    
    concurrent = False
    # flag that lets us know which mode we're running
    # concurrent = False -> sequence mode (default)
    # concurrent = True  -> concurrent mode
    
    pending = None
    # list of pending scanners to complete the process for concurrent mode.
    
    def addBlueZSignalHandlers(self):
        # Subscribe to signals
        self.bus.add_signal_receiver(
            self.device_found, # callback
            'DeviceFound', # signal name
            'org.bluez.Adapter', # interface name
            'org.bluez', # sender bus name
            path_keyword='path') #
        
        self.bus.add_signal_receiver(
            self.property_changed, # callback
            'PropertyChanged', # signal name
            'org.bluez.Adapter', # interface name
            'org.bluez', # sender bus name
            path_keyword='path') #

    def addRemoteScannerSignalHandlers(self):
        # call back for remote scanner
        self.bus.add_signal_receiver(
            self.property_changed, # callback
            signal_name='PropertyChanged', # signal name
            dbus_interface=remotescanner_url, # interface name
            path_keyword='path'
            )

        self.bus.add_signal_receiver(
            self.device_found, # callback
            signal_name='DeviceFound', # signal name
            dbus_interface=remotescanner_url, # interface name
            path_keyword='path'
            )


    def __init__(self, bus, listener=None, rpc=None):
        logger.debug("ScanManager created")
        self.bus = bus
        self.manager = dbus.Interface(
                                bus.get_object(const.BLUEZ, const.BLUEZ_PATH), 
                                const.BLUEZ_MANAGER
        )
        if listener is not None:
            for x in listener:
                self.exposed_addListener(x)
        self.exposed_refreshScanners()
        
        self.addBlueZSignalHandlers() # if this fails we're dead
        
        try:
            self.addRemoteScannerSignalHandlers() 
            # this may fail and we don't care
        except:
            pass
        
        self.rpc = rpc
        if self.rpc:
            self.remote_listener=rpyc.async(self.rpc.root.listener)
    
    def exposed_refreshScanners(self):
        '''
        When called by the server this method will create ScanAdapter or 
        RemoteScanAdapter for each one of our scanner dongles and then will
        create the can sequence. Letting the server know when we are ready
        to start scanning by triggering DONGLES_ADDED
        '''
        logger.debug("ScanManager refresh scanners %s" % self.scanners)
        if self.scanners is None or len(self.scanners) == 0:
            self.__dongles = dict()
            return False
        try:
            for i in self.scanners.keys():
                try:
                    path=self.manager.FindAdapter(i)
                    adapter = ScanAdapter(self.scanners[i][0], 
                    name=self.scanners[i][1], 
                    bus=self.bus, path=path)
                except Exception, err:
                    print err
                    print "trying with a remote scanner"
                    try:
                        adapter = RemoteScanAdapter(self.scanners[i][0], 
                            local=self.scanners[i][1], address=i, 
                            bus=self.bus, scanner_manager=self)
                    except Exception, err2:
                        logger.error(err2)
                        continue
                
                self.__dongles[i] = adapter
        
            self.__generateSequence()
            self.tellListenersAsync(DONGLES_ADDED)
        except Exception, err:
            print err
            raise err
    
    def __generateSequence(self):
        # we want a sequence were we interleave devices as
        # PRIORITY=2, PRIORITY=1, PRIORITY=2
        # so that dongles with higher priority will get more scanning slots
        logger.debug('scanner generating sequence')
        priority=dict()
        slots = 0
        
        for dongle in self.__dongles.itervalues(): # extract priority from __dongles
            if dongle.priority not in priority:
                priority[dongle.priority]=list()
            priority[dongle.priority].append(dongle)
            slots+=dongle.priority
        logger.debug('scanner sequence %s slots' % slots)
        
        priorities=priority.keys()
        priorities.sort(reverse=True) # this gets higher to lower priority
        __sequence = list()
        
        a=0
        for p in priorities:
            # first we add the dongles with the lowest priority
            for dongle in self.__dongles.itervalues():
                if p==dongle.priority:
                    logger.debug('slot[%s] = %s' % (a, dongle))
                    __sequence.append(dongle)
                    a+=1

        for dongle in self.__dongles.itervalues():
            if p < dongle.priority:
                logger.debug('slot[%s] = %s' % (a, dongle))
                __sequence.append(dongle)
                a+=1
        
        self.__sequence=__sequence
        self.__index = 0
        
    def exposed_setConcurrent(self, val):
        '''
        Exposed method that lets the server tell us if we should do concurrent 
        scanning (all the scanners work at the same time), or sequence scanning.
        '''
        self.concurrent = val
        logger.info("setConcurrent %s" % val)
    
    def exposed_getConcurrent(self):
        '''
        Exposed method that lets the server know if we are in concurrent or
        sequence mode (default)
        '''
        return self.concurrent

    def tellListenersAsync(self, *args, **kwargs):
        '''
        Dispatch the given signal with the given arguments to our listener.
        '''
        logger.debug("ScanManager telling listener - async: %s" % POST[args[0]])
        self.remote_listener(*args, **kwargs)
        logger.debug("ScanManager message dispatched")

    def exposed_startScanningCycle(self):
        '''
        Exposed method that lets the server decide when we start a new scan 
        cycle this means we restart our index for sequence mode, and we drop
        out our pending scans for concurrent mode
        NOTE: this isn't very reliable, we shouldn't relay this to the server
        '''
        self.tellListenersAsync(CYCLE_START)
        self.__index = 0
        if self.concurrent:
            self.pending=list()
    
    def exposed_noCampaigns(self):
        '''
            this method gets called when there was no matching campaigns on the
            server, so we tell the server until it's ready
        '''
        def request_server(self):
            self.tellListenersAsync(CYCLE_COMPLETE)
            return False
        logger.info("no campaigns, scheduling request")
        gobject.timeout_add(60*1000, request_server, self)

    def exposed_getDongles(self):
        '''
        Exposed method that lets the server know which dongles we are handling.
        '''
        out = set()
        for d in self.manager.ListAdapters():
            out.append(str(d.GetProperties()['Address']))
        return out

    def exposed_add_dongle(self, address, priority, name):
        '''
        Exposed method that lets the server tell us which dongle use with which
        settings (name and priority)
        '''
        self.scanners[address]=(priority, name)
        
    def exposed_doScan(self):
        '''
        Exposed method that tells us to start scanning.
        '''
        logger.debug('ScanManager scanning')
        
        if not self.concurrent:
            self.__do_scan_no_concurrent()
        else:
            self.__multi_scan()
        
    def __do_scan_no_concurrent(self):
        '''
        internal method that starts a sequence scan
        '''
        logger.debug("No concurrent scan")
        if self.__index < len(self.__sequence):
            try:
                logger.debug('ScanManager scanning on dongle: %s' % self.__sequence[self.__index].bt_address)
                self.__sequence[self.__index].scan()
            except Exception, err:
                logger.debug("Couldn't scan, error: %s" % err)
                self.__sequence[self.__index].endScan()
                addr = str(self.__sequence[self.__index].bt_address)
                ret=self.__rotate_dongle()
                if not ret:
                    self.tellListenersAsync(DONGLE_NOT_AVAILABLE, address=addr)

    def __multi_scan(self):
        '''
        internal method that starts a concurrent scan
        '''
        logger.debug("Concurrent scan")
        if self.pending is not None and len(self.pending) > 0:
            raise Exception("Can't do multiple concurrent scans if pending")
        self.pending = list()
        for dongle in self.__dongles.itervalues():
            logger.debug("%s scanning" % dongle.dbus_path)
            dongle.scan()
            self.pending.append(dongle.dbus_path)
    
    def __rotate_dongle(self):
        '''
        Internal method used for concurrent scan, each time each one of the
        scanners completets it's work this method gets called until there
        are no more dongles in the sequence.
        When that happens we tell the server we're done.
        '''
        self.__index += 1
        if self.__index >= len(self.__sequence):
            self.tellListenersAsync(CYCLE_COMPLETE)
            self.__index = 0
            logger.debug('ScanManager dongle cycle completed')
            return True
        logger.debug('ScanManager dongle rotated, dongle: %s' % self.__sequence[self.__index])
        return False

    def __getScannerForPath(self, path):
        '''
        Translate dbus path to scanner device
        '''
        for dongle in self.__dongles.itervalues():
            if dongle.dbus_path == path:
                return dongle
        raise Exception("Path not Valid")

    # signal callbacks      
    def device_found(self, address, values, path=None):
        '''
        This callback gets called when ever a device has been found over dbus.
        We add the result to our discovered buffer, so we then tell the server
        all the results together. With real discovery time.
        '''
        dongle = self.__getScannerForPath(path)
        if address not in dongle.found:
            dongle.found[address] = dict()
            dongle.found[address]['rssi'] = list()
            dongle.found[address]['time'] = list()
            
        if 'name' not in dongle.found[address] and 'Name' in values:
            dongle.found[address]['name']=smart_str(values['Name'])
        if  'devclass' not in dongle.found[address] and 'Class' in values:
            dongle.found[address]['devclass'] = int(values['Class'])
        
        dongle.found[address]['rssi'].append(int(values['RSSI']))
        dongle.found[address]['time'].append(time.time())
        logger.debug(
                '%s device found: %s, %s' % (
                                                path, 
                                                address, 
                                                values.get('Name', None)
                                            )
                    )
        
    def property_changed(self, name, value, path=None):
        '''
        Callback that will let us know when a scan cycle has been completed,
        or started.
        '''
        if self.__index == None:
            return
        if name=='Discovering':
            if value==0:
                logger.debug('Discovery completed for path: %s' % path)
                dongle = self.__getScannerForPath(path)
                dongle.endScan()
                if self.concurrent and dongle.dbus_path in self.pending:
                    self.pending.remove(dongle.dbus_path)
                self.discovery_completed(dongle)
                return
            else:
                logger.debug('Discovery started for path: %s' % path)
                return
        logger.debug("property_changed %s %s %s" % (name, value, path) )
    
    def exposed_cycleCompleted(self):
        '''
        Exposed method that lets the server know if our scan cycle is done.
        '''
        return self.__index == len(self.__sequence) - 1
            
    def discovery_completed(self, dongle):
        '''
        This method gets called once the scan cycle is done, or all the 
        concurrent scanners have done they're job. This is when we send
        the server all our results. And tell we're done for a new scan
        cycle
        '''
        founds = list()
        
        if dongle.found:
            for found,data in dongle.found.iteritems():
                founds.append(
                    { 
                    'address': str(found), 
                    'name': smart_str(data.get('name', None)),
                    'rssi': data['rssi'],
                    'time': data['time'],
                    'devclass': int(data.get('devclass',-1))
                    }
                )

        addr=str(dongle.bt_address)
        
        if len(founds) > 0:
            self.tellListenersAsync(FOUND_DEVICE, 
            address=str(dongle.bt_address),
            records=ByValWrapper(founds) )
        
        if not self.concurrent:
            ret=self.__rotate_dongle()
            if not ret:
                self.tellListenersAsync(
                    CYCLE_SCAN_DONGLE_COMPLETED,
                    address=addr
                )
        elif len(self.pending) == 0:
            self.tellListenersAsync(CYCLE_COMPLETE)

if __name__=='__main__':
    def listen(signal, **kwargs):
        print signal, TEXT[signal], kwargs, signals.isScannerSignal(signal)
        
        if signal == CYCLE_COMPLETE:
            loop.quit()

    import dbus
    import gobject
    import dbus.glib
    import dbus.mainloop.glib
    
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    
    manager=ScanManager(dbus.SystemBus())
    manager.remote_listener=listen
    manager.scanners['00:50:C2:7F:EF:FE']=1
    
    gobject.threads_init()
    dbus.glib.init_threads()
    loop=gobject.MainLoop()
    manager.exposed_startScanningCycle()
    loop.run()




# -*- coding: utf-8 -*-
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from django.db import transaction
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.template.loader import get_template
from django.utils import simplejson
from django.utils.encoding import smart_unicode
from django.views.generic import list_detail
from django.utils.translation import ugettext as _
from django.contrib.admin.views import decorators
from django.db import transaction
from django.conf import settings

from datetime import datetime
from net.aircable.openproximity.pluginsystem import pluginsystem
from net.aircable.utils import getLogger
logger = getLogger(__name__)

from re import compile
from mimetypes import guess_type as guess_mime

from models import *
from forms import *
import rpyc, os, time

from django import forms
from timezones.forms import TimeZoneField
from timezones import zones
import pytz

def mycmp(x, y):
    a=int(x[4:9])
    b=int(y[4:9])
    return cmp(a,b)


class MyTimeZoneField(TimeZoneField):
    def __init__(self, *args, **kwargs):
        a=dict(zones.PRETTY_TIMEZONE_CHOICES).values()
        a.sort(mycmp)
        tz=tuple(zip(a, a))
        kwargs['choices'] = tz
        super(MyTimeZoneField, self).__init__(*args,**kwargs)
    
    def clean(self, value, *args, **kwargs):
        value=value[11:]
        try:
            return pytz.timezone(value)
        except:
            raise forms.ValidationError(_("Invalid option"))

class UserForm(forms.Form):
    username = forms.CharField(max_length=100, label=_("Username:"))
    password1 = forms.CharField(
                    label=_("Password:"), 
                    max_length=100,
                    widget=forms.widgets.PasswordInput)
    password2 = forms.CharField(
                    label=_("Type Again"), 
                    max_length=100,
                    widget=forms.widgets.PasswordInput)
    email = forms.EmailField()
    time_zone = MyTimeZoneField()

    def clean(self):
        cleaned_data = self.cleaned_data
        pass1 = cleaned_data.get("password1", None)
        pass2 = cleaned_data.get("password2", None)
        if pass1 != pass2:
            self._errors["password2"]=_("Passwords don't match")
            if "password2" in cleaned_data:
                del cleaned_data["password2"]

        return cleaned_data

@transaction.commit_manually
def db_ready():
    out = False
    try:
        from models import BluetoothDongle, UserProfile
        BluetoothDongle.objects.count()
        out = UserProfile.objects.count() > 0
    except:
        pass
    finally:
        transaction.rollback()
    return out

def CreateDB():
    from django.core import management
    import rpyc
    os.system('mkdir -p %s' % settings.STORE_PATH)
    os.system('mkdir -p %s' % settings.MEDIA_PATH)
    try:
        server=rpyc.connect('localhost', 8010)
        server.root.Lock()
        logger.info("database locked")
    except:
        pass

    management.call_command("syncdb", interactive=False)
    
    management.call_command("migrate", all_apps=True, interactive=False)
    
    try:
        server=rpyc.connect('localhost', 8010)
        server.root.Unlock()
        server.root.restart()
    except:
        pass


def CreateAdmin(username, password, email, time_zone):
    from django.contrib.auth.models import User
    if len(User.objects.filter(username=username))>0:
        admin = User.objects.get(username=username)
    else:
        admin = User.objects.create_superuser(username, email, password)
        admin.save()
    profile = admin.userprofile_set.create()
    profile.timezone = time_zone
    profile.save()

def CreateSecretKey():
    from random import choice, seed
    seed()
    key = ''.join([choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])
    A=open(settings.OPENPROXIMITY_CONFIG_FILE, 'a+')
    A.write('[django]\n')
    A.write('secret_key=%s\n' % key)
    A.write('\n')
    A.close()

def index(request):
    if request.method == 'POST':
        form = UserForm(request.POST)
        if form.is_valid():
            CreateSecretKey()
            CreateDB()
            
            CreateAdmin( 
                form.cleaned_data['username'],
                form.cleaned_data['password1'],
                form.cleaned_data['email'],
                form.cleaned_data['time_zone']
            )
            return render_to_response("op/setup.html", { 
                "done": True,
            }, context_instance=RequestContext(request))
    else:
        form = UserForm()

    return render_to_response("op/setup.html", {
            "db_ready": db_ready(),
            "user": form,
        }, context_instance=RequestContext(request))



# -*- coding: utf-8 -*-
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from django.conf import settings
from django.utils.encoding import smart_unicode
from django.utils.translation import ugettext as _
from net.aircable.openproximity.signals import scanner as signals
from net.aircable.utils import getLogger
logger = getLogger(__name__)
from openproximity.models import *
from rpyc import async
from common import found_action, is_known_dongle

import time

def handle(signal, *args, **kwargs):
    logger.info("scanner signal: %s" % signals.TEXT[signal])
    logl = LogLine()
    logl.content += signals.TEXT[signal]

    if signal == signals.DONGLES_ADDED:
        logger.info("Dongles initializated")
        cycle_completed()
    elif signal == signals.NO_DONGLES:
        logger.error("NO SCANNER DONGLES!!!")
    elif signal == signals.DONGLE_NOT_AVAILABLE:
        logger.error("DONGLE NOT AVAILABLE %s" % kwargs['address'])
        logl.content += " " + kwargs['address']
        do_scan()
    elif signal == signals.CYCLE_SCAN_DONGLE_COMPLETED:
        logger.info("DONGLE DONE WITH SCAN %s" % kwargs['address'])
        logl.content += " " + kwargs['address']
        do_scan()
    elif signal == signals.CYCLE_COMPLETE:
        cycle_completed()
    elif signal == signals.CYCLE_START:
        pass
    elif signal == signals.CYCLE_SCAN_DONGLE:
        logl.content += " " + kwargs['address']
        started(kwargs['address'])
    elif signal == signals.FOUND_DEVICE:
        logl.content += " " + kwargs['address']
        addrecords(kwargs['address'], 
            kwargs['records']
        )
    else:
        logger.error("unknown signal")
        raise Exception("Not known signal")
    
    logl.save()

def started(address):
    logger.info('scan_started %s' % address)
    dongle = ScannerBluetoothDongle.objects.get(address=address)
    record = DeviceRecord()
    record.action = signals.CYCLE_SCAN_DONGLE
    record.dongle = dongle
    record.save()

def get_dongles(dongles):
    def create_new_discovered_dongle(address, settings):
        logger.info("going to setup as scanner")
        priority = settings.get('value', 1)
        enabled = settings.get('enable', True)
        name = settings.get('name', _("Auto Discovered Dongle"))
        obj, created = ScannerBluetoothDongle.objects.get_or_create(
            address=address, 
            defaults={
                'priority': priority,
                'enabled': enabled,
                'name': name
            })
        logger.debug("%s %s[%s]" % (address, name, priority))

    def internal_get_dongles(address):
        dongle = ScannerBluetoothDongle.objects.get(address=address)
        logger.info("%s is a scanner dongle" % address)
            
        if dongle.enabled:
            yield (address, dongle.priority, dongle.name)
                
        if dongle.remote_dongles.count() > 0:
            logger.info("We have remote dongles available")
            for remote in dongle.remote_dongles.all():
                if remote.enabled:
                    yield ( remote.address, remote.priority, dongle.address )

    out = list()
    
    for address in dongles:
        try:
            if not is_known_dongle(address, ScannerBluetoothDongle):
                logger.info("dongle not known yet %s" % address)
                setting = settings.GETSCANNERDONGLE(address)
                if setting:
                    create_new_discovered_dongle(address, setting)
            out.extend(internal_get_dongles(address))
        except Exception, err:
            logger.exception(err)
    return out

def do_scan():
    from openproximity.rpc.server import OpenProximityService
    logger.info("start scan")
    scanner = OpenProximityService.getScanner()
    scanner.doScan()

def cycle_completed():
    from openproximity.rpc.server import OpenProximityService
    logger.info("scanner_cycle_complete")
    camps = getMatchingCampaigns(enabled=True)
    scanner = OpenProximityService.getScanner()
    if len(camps)==0:
        logger.info("no campaigns, no more scanning")
        # tell the rpc scanner to ping us periodically until there's
        # a new campaign available
        scanner.noCampaigns()
        return

    concurrent = False
    for camp in camps:
        if camp.concurrent_scanning:
            concurrent = True
            break

    logger.info("starting scan cycle")
    scanner.setConcurrent(concurrent)
    scanner.startScanningCycle()
    scanner.doScan()

uploaded = set()

def handle_addrecord(remote_, dongle):
    from openproximity.rpc.server import OpenProximityService
    address = remote_['address']
    name = smart_unicode(remote_['name']) if remote_['name'] else None

    logger.info("handle_addrecord %s" % address)
    remote=RemoteDevice.getRemoteDevice(
        address=address, 
        name=name, 
        devclass=remote_['devclass']
    )
    record = RemoteBluetoothDeviceFoundRecord()
    record.action = signals.FOUND_DEVICE
    record.dongle = dongle
    record.remote = remote
    record.remote.save() # update last seen
    record.setRSSI(remote_['rssi'])
    
    if not isinstance(dongle, RemoteScannerBluetoothDongle):
        if name is not None and record.remote.name!=name:
            record.remote.name = name
            record.remote.save()
        if remote_['devclass'] != -1 and \
                    record.remote.devclass!=remote_['devclass']:
            record.remote.devclass = remote_['devclass']
            record.remote.save()

    logger.debug(record)
    logger.debug(record.remote)

    record.save() # this will set the time field to now
    
    logl = LogLine()
    logl.content="Found: %s %s %s" % (
        record.remote.address, 
        record.remote.name,
        record.getRSSI())
    logl.save()
    
    if not OpenProximityService.isPending(address):
        return found_action(address, record, dongle=dongle)

    return True
    
def addrecords(address, records):
    logger.info('addrecords for dongle %s' % address)
    try:
        dongle = RemoteScannerBluetoothDongle.objects.get(address=address)
        logger.info("remote dongle")
    except:
        logger.info("local dongle")
        dongle = ScannerBluetoothDongle.objects.get(address=address)    

    for i in records:
        handle_addrecord(i, dongle)

    return True




# -*- coding: utf-8 -*-
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
# Create your views here.

from django.db import transaction
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.template.loader import get_template
from django.utils import simplejson
from django.utils.encoding import smart_unicode
from django.views.generic import list_detail
from django.contrib.admin.views import decorators

from django.conf import settings

from datetime import datetime
from net.aircable.openproximity.pluginsystem import pluginsystem
from net.aircable.utils import getLogger
logger = getLogger(__name__)

from re import compile
from mimetypes import guess_type as guess_mime

from models import *
from forms import *
import rpyc, os, time, setup

def add_record_accepted(request):    
    return HttpResponse('Recorded\n')

def add_record(request):
    if not request.method == 'POST':
        raise Http404("I only undertand POST")
    
    time = request.POST.get('time', None)
    address_ = request.POST.get('address', None)
    action_ = request.POST.get('action', None)
    action_extra = request.POST.get('action_extra', None)
    server_address_ = request.POST.get('server_address', None)
    
    if address_ is None or action_ is None:
        raise Http404("Missing required arguments")
    
    record = DeviceRecord()
    record.action = get_object_or_404(RemoteBluetoothAction, short_name=action_)
    
    if server_address_ is not None:
        try:
            record.dongle = BluetoothDongle.objects.get(
                    address__exact=server_address_)
        except:
            pass        
    
    record.time = time or None
    record.address = address_
    record.action_extra = action_extra
    record.save()
    
    return HttpResponse('Recorded\n')

AIRCABLE_MAC=['00:50:C2', '00:25:BF']
ADDRESS_MAC=compile("([0-9A-F]{2}\:){5}([0-9A-F]{2})")

def isAIRcable(address):
    return address[:8].upper() in AIRCABLE_MAC
    
def add_dongle(address, name, scanner, uploader, scanner_pri=1, uploader_max=7):
    logger.info("add_dongle %s %s %s %s %s %s" % 
        (address, name, scanner, uploader, scanner_pri, uploader_max)
    )
    search=ScannerBluetoothDongle.objects.filter(address=address)
    
    if scanner and search.count()==0:
        rec = ScannerBluetoothDongle()
        rec.address = address
        rec.name = name
        rec.priority = scanner_pri
        rec.enabled = True
        rec.save()
        logger.debug("scanner %s" % rec)
    
    if search.count()>0:
        rec=search.get()
        rec.enabled = scanner == True
        rec.priority = scanner_pri
        rec.save()
    
    search=UploaderBluetoothDongle.objects.filter(address=address)
    
    if uploader and search.count()==0:
        rec = UploaderBluetoothDongle()
        rec.address = address
        rec.name = name
        rec.max_conn = uploader_max
        rec.enabled = True
        rec.save()
        logger.debug("uploader %s" % rec)
    
    if search.count()>0:
        rec=search.get()
        rec.enabled = uploader == True
        rec.save()

@decorators.staff_member_required
def configure_campaign(request, name=None):
    logger.info("configure_campaign %s" % name)
    form = CampaignForm()
    logger.debug(form.as_table())

    return render_to_response('op/campaign_form.html',
        { 
            'form':  form,
        }, context_instance=RequestContext(request)
    )

@decorators.staff_member_required
def configure_dongle(request, address=None):
    logger.info("configure_dongle %s" % address)
    
    errors = []
    messages = []
    form = None
    if request.method == "POST":
        form=DongleForm(request.POST)
        if form.is_valid():
            cd=form.cleaned_data
            add_dongle(
                cd['address'],
                cd['name'],
                cd["scan"],
                cd["upload"],
                cd["scan_pri"],
                cd["upload_max"],
            )
            return HttpResponseRedirect('/')
            #messages.append("Dongle Configured")

    scanner = None
    scanner_pri = 1
    uploader = None
    uploader_max = 7
    
    name = "OpenProximity 2.0"
    
    search=BluetoothDongle.objects.filter(address=address)
    if search.count()>0:
        dongle = search.all()[0]
        name=dongle.name
    
    search=ScannerBluetoothDongle.objects.filter(address=address)
    if search.count() > 0:
        scanner = True
        scanner_pri=search.get().priority
    
    search=UploaderBluetoothDongle.objects.filter(address=address)
    if search.count() > 0:
        uploader = True
        uploader_max=search.get().max_conn

    form = form or DongleForm( initial = { 
                                    'address': address, 
                                    'name': name,
                                     'scan': scanner,
                                     'scan_pri': scanner_pri,
                                     'upload': uploader,
                                     'upload_max': uploader_max})

    return render_to_response('op/dongle_form.html',
        { 
            'form':  form,
            'messages': messages,
        }, context_instance=RequestContext(request))

@decorators.staff_member_required
def server_rpc_command(request, command):
    server=rpyc.connect('localhost', 8010)
    func=getattr(server.root, command, None)
    if func is not None:
        rpyc.async(func)()
    return HttpResponseRedirect('/')
    
def grab_file(request, file):
    logger.info("grab_file %s" % file)
    file = CampaignFile.objects.get(file=file).file
    mime = guess_mime(file.name)
    return HttpResponse(file.read(), mimetype=mime[0] )

@decorators.staff_member_required
def stats_restart(request):
    '''
    Delete statistics, we do drop table, not the recommended way but damn
    effective.
    '''
    from django.core import management
    from django.db import connection, models
    from django.core.management.color import no_style
    from django.core.management import sql

    cursor = connection.cursor()
    logger.info("stats restart")

    # this tables are not going to be deleted
    tables = [ 'openproximity_bluetoothdongle',
        'openproximity_campaignfile',
        'openproximity_marketingcampaign',
        'openproximity_remotescannerbluetoothdongle',
        'openproximity_scannerbluetoothdongle',
        'openproximity_uploaderbluetoothdongle',
        'openproximity_generalsetting',
        'openproximity_userprofile'
        ]
    model  = models.get_app('openproximity')
    drop = ""
    
    drop_table = sql.sql_delete(model, no_style(), connection)
    
    for line in drop_table:
        table_name = line.split()[2].replace('"', '').replace(';','')
    
        if line.startswith('DROP TABLE'):
            # we don't want to loose settings    
            if table_name not in tables:
                drop+="DROP TABLE %s;\n" % table_name
            
            elif line.find('CREATE INDEX') > -1:
                drop += "DROP INDEX %s;\n" % table_name
    try:
        server=rpyc.connect('localhost', 8010)
        server.root.Lock()
        logger.info("database locked")
    except:
        pass

    logger.info("about to drop")
    for line in drop.splitlines():
        try:
            logger.debug(line)
            connection.cursor().execute(line)
        except Exception, err:
            logger.error("%s failed" %line)
            logger.exception(err)

    logger.info("allowing plugins to drop statistic it's tables")

    for plugin in pluginsystem.get_plugins('statistics_reset'):
        try:
            getattr(plugin.module,'statistics_reset')(connection)
        except Exception, err:
            logger.error("plugin failed to reset statistics %s" % plugin)
            logger.exception(err)

    logger.info("calling syncdb")
    management.call_command('syncdb', migrate_all=True)

    try:
        server=rpyc.connect('localhost', 8010)
        server.root.Unlock()
        server.root.restart()
    except:
        pass
    
    logger.info("database unlocked")
    
    return HttpResponse("DELETE COMPLETE")
    
def dongle_information(dongle):
    a=dict()
    a['address'] = dongle
                
    search = ScannerBluetoothDongle.objects.filter(address=dongle)
    a['isScanner'] = search.count()>0
    if search.count()>0:
        a['scan_enabled'] = search.get().enabled == True
        a['scan_pri'] = search.get().priority
        
    search = UploaderBluetoothDongle.objects.filter(address=dongle)
    a['isUploader'] = search.count()>0      
    if search.count()>0:
        a['upload_enabled'] = search.get().enabled == True
        a['max_conn'] = search.get().max_conn
    
    return a

def generate_rpc_information():
    # generate rpc information
    rpc = dict()
    rpc['running'] = None
    try:
        server=rpyc.connect('localhost', 8010)
        rpc['running'] = server is not None
        rpc['uploaders'] = server.root.getUploadersCount()
        rpc['scanners'] = server.root.getScannersCount()
        rpc['dongles'] = [ dongle_information(a) for a in 
                server.root.getDongles()]
    except Exception, err:
        rpc['error'] = repr(err)

    return rpc

def generate_stats():
    # generate stastics information
    stats = dict()
    try:
        stats['seen'] = dict()
        stats['seen']['total'] = RemoteDevice.objects.count()
        stats['seen']['perdongle'] = list()
        for a in ScannerBluetoothDongle.objects.all():
            b=dict()
            b['address'] = a.address
            b['count'] = RemoteBluetoothDeviceFoundRecord.objects.\
                    filter(dongle=a).count()
            stats['seen']['perdongle'].append( b )
        stats['valid'] = RemoteBluetoothDeviceSDP.objects.count()
    
        accepted = RemoteBluetoothDeviceFilesSuccess.objects.count()
        non_accepted = RemoteBluetoothDeviceFilesRejected.objects.count()
    
        a=RemoteBluetoothDeviceFilesRejected.objects
        for ret in TIMEOUT_RET:
            a=a.exclude(ret_value=ret)
            
        rejected = a.count()
        
        stats['accepted'] = accepted
        stats['rejected'] = rejected
        stats['timeout'] = non_accepted-rejected
        stats['tries'] = accepted+non_accepted

    except Exception, err:
        stats['error'] = repr(err)

    return stats

def index(request):
    # check if DB needs to be created
    if not setup.db_ready():
        return setup.index(request)

    # generate rpc information
    rpc = generate_rpc_information()
    
    # generate stastics information
    stats = generate_stats()

    return render_to_response("op/index.html", {
                                               "rpc": rpc,
                                               "camps": getMatchingCampaigns(),
                                               "stats": stats,
            }, 
            context_instance=RequestContext(request))

def rpc_stats(request):
    stats = generate_stats()
    return HttpResponse(
        simplejson.dumps(stats), 
        content_type="application/json")

def rpc_info(request):
    info = generate_rpc_information()
    return HttpResponse(
        simplejson.dumps(info), 
        content_type="application/json")

def smart_group(group):
    for g in group:
        line=dict()
        for key,val in g.iteritems():
            line[key]=smart_unicode(val)
        yield line

def rpc_last_seen(request):
    #get a json list of the devices seen in the last 30 minutes
    secs=time.time()-15*60
    start=datetime.fromtimestamp(secs)
    objs=RemoteDevice.objects.filter(
        last_seen__gte=start).order_by('address')

    if objs.count() > 0:
        return HttpResponse(
            simplejson.dumps(
                list(smart_group(
                    objs.values(
                    'address', 
                    'name', 
                    'last_seen', 
                    'devclass')
                    )
                )),
            content_type="application/json")

    return HttpResponse(
        simplejson.dumps({}),
        content_type="application/json")

def rpc_device_info(request):
    addr=request.GET.get('address')

    out=dict()
    out['accepted'] = RemoteBluetoothDeviceFilesSuccess.objects.\
                        filter(remote__address=addr).count()

    non_accepted = RemoteBluetoothDeviceFilesRejected.objects.\
                        filter(remote__address=addr).count()
    a=RemoteBluetoothDeviceFilesRejected.objects.filter(remote__address=addr)
    for ret in TIMEOUT_RET:
        a=a.exclude(ret_value=ret)

    out['rejected'] = a.count()
    out['timeout'] = non_accepted-out['rejected']
    out['tries'] = out['accepted']+non_accepted
    if out['tries'] > 0:
        out['valid'] = RemoteBluetoothDeviceSDP.objects.\
                            filter(remote__address=addr).count()>0
    else:
        out['valid'] = 'N/D'

    return HttpResponse( simplejson.dumps(out), content_type="application/json")

# changes from Manuel Coli
def rpc_campaign_info(request):
    id=request.GET.get('campaign')

    out=dict()
    out['accepted'] = RemoteBluetoothDeviceFilesSuccess.objects.filter(campaign=id).count()

    non_accepted = RemoteBluetoothDeviceFilesRejected.objects.filter(campaign=id).count()
    a=RemoteBluetoothDeviceFilesRejected.objects.filter(campaign=id)
    for ret in TIMEOUT_RET:
        a=a.exclude(ret_value=ret)

    out['rejected'] = a.count()
    out['timeout'] = non_accepted-out['rejected']
    out['tries'] = out['accepted']+non_accepted

    return HttpResponse(
        simplejson.dumps(out),
        content_type="application/json"
    )

def rpc_campaign_device(request):
    id=request.GET.get('campaign')
    out=dict()
    objs= RemoteBluetoothDeviceFileTry.objects.filter(campaign=id) 
    device = RemoteDevice.objects.filter(id__in=objs.values('remote'))

    if device.count() > 0:
        return HttpResponse(
            simplejson.dumps(
                list(smart_group(
                    device.values(
                        'id',
                        'address', 
                        'name', 
                        'last_seen', 
                        'devclass')
                    )
                )),
            content_type="application/json"
        )

    return HttpResponse(
        simplejson.dumps(out),
        content_type="application/json"
    )

def rpc_campaign_tot(request):

    out=dict()
    out['accepted'] = RemoteBluetoothDeviceFilesSuccess.objects.count()

    non_accepted = RemoteBluetoothDeviceFilesRejected.objects.count()
    a=RemoteBluetoothDeviceFilesRejected.objects.all()
    for ret in TIMEOUT_RET:
        a=a.exclude(ret_value=ret)

    out['rejected'] = a.count()
    out['timeout'] = non_accepted-out['rejected']
    out['tries'] = out['accepted']+non_accepted

    return HttpResponse(
        simplejson.dumps(out),
        content_type="application/json"
    )

def rpc_campaign_device_accepted(request):
    id=request.GET.get('campaign')

    out=dict()
    objs= RemoteBluetoothDeviceFilesSuccess.objects.filter(campaign=id)
    device = RemoteDevice.objects.filter(id__in=objs.values('remote'))
    
    if device.count() > 0:
        return HttpResponse(
            simplejson.dumps(
                list(smart_group(
                    device.values(
                        'id',
                        'address', 
                        'name', 
                        'last_seen', 
                        'devclass')
                    )
                )),
            content_type="application/json"
        )

    return HttpResponse(
        simplejson.dumps(out),
        content_type="application/json"
    )

def rpc_campaign_device_rejected(request):
    id=request.GET.get('campaign')
    out=dict()
    objs= RemoteBluetoothDeviceFilesRejected.objects.filter(campaign=id)
    device = RemoteDevice.objects.filter(id__in=objs.values('remote'))

    if device.count() > 0:
        return HttpResponse(
            simplejson.dumps(
                list(smart_group(
                    device.values(
                        'id',
                        'address', 
                        'name', 
                        'last_seen', 
                        'devclass')
                    )
                )),
            content_type="application/json"
        )

    return HttpResponse(
        simplejson.dumps(out),
        content_type="application/json"
    )

def rpc_campaign_device_ignored(request):
    id=request.GET.get('campaign')
    out=dict()
    objs= RemoteBluetoothDeviceSDPTimeout.objects.filter(campaign=id)
    device = RemoteDevice.objects.filter(id__in=objs.values('remote'))

    if device.count() > 0:
        return HttpResponse(
            simplejson.dumps(
                list(smart_group(
                    device.values(
                        'id',
                        'address', 
                        'name', 
                        'last_seen', 
                        'devclass')
                    )
                )),
            content_type="application/json"
        )

    return HttpResponse(
        simplejson.dumps(out),
        content_type="application/json"
    )

def stats_restart_campaign(request):
    # this method is still not complete
    id=request.GET.get('campaign')
    
    out=dict()

    objs= RemoteBluetoothDeviceFileTry.objects.filter(campaign=id).delete() 
    
    return HttpResponse("DELETE CAMPAIGN COMPLETE")
# end of changes from Coli

RPC_COMMANDS={
    'stats':                    rpc_stats,
    'info':                     rpc_info,
    'last-seen':                rpc_last_seen,
    'device-info':              rpc_device_info, 
    # methods from Coli
    'campaign-info':            rpc_campaign_info,
    'campaign-tot':             rpc_campaign_tot,
    'campaign-device':          rpc_campaign_device,
    'campaign-device-accepted': rpc_campaign_device_accepted,
    'campaign-device-rejected': rpc_campaign_device_rejected,
    'campaign-device-ignored':  rpc_campaign_device_ignored,
#    'delete-statistics':       stats_restart,
    'delete-statistics-campaign':stats_restart_campaign,
}

def rpc_command(request, command):
    f = RPC_COMMANDS.get(command, None)
    logger.info("rpc_command %s" % command)
    return f(request) if f else HttpResponse("Non Valid Command")

def last_seen(request):
    return render_to_response(
        "op/last_seen.html", 
        {}, 
        context_instance=RequestContext(request)
    )



# -*- coding: utf-8 ; Mode: python; tab-width: 4 ; indent-tabs-mode: nil -*-
# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import dbus, dbus.service, time, os, rpyc
import net.aircable.const as const
import net.aircable.utils as utils
import net.aircable.openproximity.signals as signals
import net.aircable.sdp as sdp
from net.aircable.openproximity.signals.uploader import *
from net.aircable.utils import *
from net.aircable.wrappers import Adapter
from rpyc.lib.compat import ByValWrapper
from functools import partial
from agent import Agent, PATH
import async

logger = utils.getLogger(__name__)

from django.conf import settings

MEDIA_ROOT = settings.MEDIA_ROOT
TIMEOUT = os.environ.get('TIMEOUT', '20')

class UploadAdapter(Adapter):
    '''
    Class to wrap an uploader. All the process is done async, this means we
    tell the upload process to start and then wait (active) until it gives us
    back some callbacks.
    '''
    
    queue = None
    # not used ?
    
    max_uploads = None
    # max aumount of simultaneous uploads
    
    slots = None
    # upload slots
    
    manager = None
    # instance of our Uploader Manager
    
    loop = None
    # reference to the running gobject loop

    def __init__(self,
                    manager, 
                    max_uploads = 7,
                    loop = None,
                    *args, 
                    **kwargs):
        Adapter.__init__(self, *args, **kwargs)

        if not self.is_aircable:
            # Ok you got me, this is the second piece you need to remove
            # MN but don't tell anyone.
            raise Exception("Can't use non AIRcable dongle as uploaders")

        self.max_uploads = max_uploads
        self.slots = dict( [ (i, None) for i in range(max_uploads) ] )
        self.manager = manager
        self.loop = loop
        self.hci_path = self.dbus_path.split("/")[-1]
        for i in range(0, max_uploads):
            Agent(self.bus, self.hci_path)
        logger.debug("Initializated UploaderDongle")

    def completed(self, target):
        ''' 
        free up our uploading slot
        '''
        #target.cleanup()
        self.slots[target.slot] = None
        logger.info("slot %s is now free" % target.slot)

    def FileUploaded(self, target, *args, **kwargs):
        '''
        Callback that lets us know when an upload process completed succesfully.
        '''
        logger.info("File uploaded %s" % target.target)
        
        self.manager.tellListeners(
            signal = FILE_UPLOADED,
            dongle = self.bt_address,
            address = str(target.target),
            port = target.channel,
            files = ByValWrapper(target.files)
        )
        self.completed(target)

    def FileFailed(self, target, retcode, stdout, stderr, *args, **kwargs):
        '''
        Callback that lets us know when an upload process completed with a 
        failure.
        '''
        logger.info("File Failed %s" % target.target)
        self.manager.tellListeners(
            signal=FILE_FAILED,
            address=str(target.target),
            dongle=self.bt_address,
            port=target.channel,
            ret = ByValWrapper(retcode),
            files = ByValWrapper(target.files), 
            stdout = stdout, 
            stderr = stderr,
            timeout = TIMEOUT
        )
        self.completed(target)

    def ChannelResolved(self, target, channel):
        '''
        Callback that let us know when the channel for the remote deivce has 
        been resolved correctly.
        '''
        logger.info("ChannelResolved %s -> %s" % ( target.target, channel ))
        self.manager.tellListeners(
            signal = SDP_RESOLVED,
            dongle = self.bt_address,
            address = str(target.target),
            port = channel
        )
        target.channel = channel
        self.completed(target)

    def do_upload(self, target):
        '''
        This function is called once we have resolved the channel.
        Is public available so we can force upload to a certain channel.
        '''
        target.SendFiles(
            channel=target.channel, 
            files=[ os.path.join(MEDIA_ROOT, f[0]) for f in target.files ],
            service = target.service,
            reply_callback=self.FileUploaded,
            error_callback=self.FileFailed,
        )
    
    def ServiceNotProvided(self, target, error, state, connected):
        '''
        Callback that will get call when channel resolving failed.
        '''
        logger.info("ServiceNotProvided %s %s %s" % (target.target, error, 
                                                     connected))
        signal = SDP_NORECORD if connected else SDP_TIMEOUT 
        
        self.manager.tellListeners(
            signal = signal,
            dongle = self.bt_address,
            address = str(target.target)
        )
        
        self.completed(target)

    def getSlot(self):
        '''
        Function that will get called when a new upload needs an upload slot.
        If none is available we raise an Exception.
        '''
        for slot in self.slots:
            if not self.slots[slot]:
                logger.debug("found slot %s" % slot)
                return slot
        raise Exception("No slot available")

    def resolveService(self, uuid, address = None, target = None):
        assert address or target
        if not target:
            target = async.UploadTarget( self.dbus_interface, address, self.bus, 
                                     loop=self.loop)
            target.slot = self.getSlot()
            target.uuid = uuid
    
        target.ResolveChannel(uuid, self.ChannelResolved, 
                              self.ServiceNotProvided)

    def pairSuccess(self, target = None):
        '''
        Callback that will get call when pairing has been successfull.
        '''
        logger.info("Pairing success %s" % target.target)
        signal = PAIR_SUCCESS 
        
        self.manager.tellListeners(
            signal = signal,
            dongle = self.bt_address,
            address = str(target.target)
        )
        self.completed(target)

    def pairFailed(self, error=None, target=None, connected=False, 
                   exception=None):
        '''
        Callback that will get call when pairing has failed
        '''
        logger.info("Pairing failed %s %s" % (target.target, error))
        signal = PAIR_TIMEOUT if not connected else PAIR_REJECT
        
        self.manager.tellListeners(
            signal = signal,
            dongle = self.bt_address,
            address = str(target.target),
            reason = str(error),
            exception = str(exception)
        )
        self.completed(target)
        

    def pairDevice(self, address=None, target=None):
        assert address or target
        if not target:
            target = async.UploadTarget( 
                self.dbus_interface, 
                address, 
                self.bus,
                loop=self.loop)
            target.slot = self.getSlot()

        try:
            logger.debug("pairing to address %s", address)
            agent = Agent.getAgent(self.hci_path)
            target.PairDevice(address, 
                              agent.path,
                              self.pairSuccess, 
                              self.pairFailed)
        except Exception, err:
            logger.exception(err)
            raise err


    def upload(self, files, target, uuid, service, channel=None):
        '''
        Function that gets called by the manager when it wants to do an upload.
        If no channel is provided then it will start an sdp resolving for the 
        service uuid.
        '''
        logger.debug("got an upload request %s" % target)
        sl = self.getSlot()
    
        target = async.UploadTarget(
                        self.dbus_interface, 
                        target, 
                        self.bus, 
                        loop=self.loop)
    
        target.files = files
        target.slot = sl
        target.service = service
        target.uuid = uuid
    
        if not channel:
            self.resolveService(uuid, target = target) 
        else:
            target.channel=int(channel)
            logger.debug("Using fixed channel %s" % channel)
            self.do_upload(target)

class UploadManager:
    '''
    A manager that will handle the work for the different dongles we have 
    connected.
    '''

    __dongles = dict()
    # dongles we're working with (UploadAdapter instances)

    bus = None
    # a connection to dbus

    manager = None
    # an instance of the BlueZ manager

    __sequence = list()
    # when multiple dongles are available then we try to do balance loading
    # by sending requests to each the dongles one at the time.

    __index = None
    # an index to know which dongle we used last.

    uploaders = dict()
    # dongles the server told us to use, this dict has address and priority

    def __init__(self, bus, rpc=None, loop=None):
        logger.debug("UploadManager created")
        self.bus = bus
        self.manager = dbus.Interface(bus.get_object(const.BLUEZ, 
                                                     const.BLUEZ_PATH), 
                                      const.BLUEZ_MANAGER)
        self.rpc = rpc
        self.loop = loop
        if self.rpc:
            self.remote_listener=rpyc.async(self.rpc.root.listener)
        
    def exposed_refreshUploaders(self):
        '''
        Exposed method that lets the server tell us when it sent us all the 
        dongles so we can create a sequence of them.
        '''
        logger.debug("UploadManager refresh uploaders %s" % self.uploaders)
        if self.uploaders is None or len(self.uploaders) == 0:
            self.__dongles = dict()
            self.tellListeners(NO_DONGLES)
            return False

        for i in self.uploaders.keys():
            adapter = UploadAdapter(
                self,
                self.uploaders[i][0],
                name=self.uploaders[i][1],
                bus=self.bus, 
                path=self.manager.FindAdapter(i)
            )
            self.__dongles[i] = adapter
        
        self.tellListeners(DONGLES_ADDED)
        self.__generateSequence()
        return True
    
    def __generateSequence(self):
        '''
        Internal method to generate a sequence of dongles.
        '''
        logger.debug('uploaders generating sequence')
        priority=dict()
        slots = 0
        __sequence=list()
        for dongle in self.__dongles.itervalues():
            __sequence.append(dongle)
                
        self.__sequence=__sequence
        self.__index = 0

    def exposed_getDongles(self):
        '''
        Exposed method that lets the server know which dongles we handle.
        (not used)
        '''
        def internal_getDongles(self):
            for d in self.manager.ListAdapters():
                yield str(d.GetProperties()['Address'])    
        return list(internal_getDongles(self))

    def tellListeners(self, *args, **kwargs):
        '''
        Tell our listener we have a signal with some arguments
        '''
        logger.debug( "UploadManager telling listener: %s, %s" %  ( 
            str(args), str(kwargs) ))
        self.remote_listener(*args, **kwargs)
        logger.debug("UploadManager signal dispatched")

    def __rotate_dongle(self):
        '''
        Once each upload request gets into our queue we have to go to the next 
        dongle in the sequence.
        '''
        if len(self.__sequence) == 1:
            return
            
        self.__index += 1
        if self.__index >= len(self.__sequence):
            self.__index = 0
        self.tellListeners(
            CYCLE_UPLOAD_DONGLE, 
            address=str(self.__sequence[self.__index].bt_address)
        )
        logger.debug(
            'UploadManager dongle rotated, dongle: %s' % 
            self.__sequence[self.__index]
        )

    def exposed_upload(self, files, target, service='opp', dongle_name=None, 
            channel=None, uploader=None):
        '''
        Exposed method that lets the server tell us it has an upload request 
        for us.
        '''
        try:
            dongle = None
            if uploader:
                logger.info("forcing dongle %s" % uploader)
                for d in self.__sequence:
                    if d.bt_address.lower() == uploader.lower():
                        dongle = d
                        logger.info("found dongle %s" % dongle)
                        break

            if not dongle:
                if self.__index is None:
                    self.__index = 0
                dongle=self.__sequence[self.__index]
            uuid = const.UUID[service]

            logger.debug("about to set dongle name to %s" % dongle_name)
            if dongle_name:
                dongle.dbus_interface.SetProperty('Name', dongle_name)

            logger.debug("uploading %s %s %s %s" % ( files, target, uuid, 
                                                     channel ) )

            for file_, fk in files:
                f = os.path.join(MEDIA_ROOT, file_)
                d = os.path.dirname(f)
                if not os.path.isdir(d) or \
                        os.path.basename(f) not in os.listdir( d ):
                    logger.debug("grabbing file %s" % d)
                    os.system('mkdir -p %s' % d )
                    A=file(f, 'w')
                    A.write(self.rpc.root.getFile(file_))
                    A.close()

            logger.debug('adding to queue')
            dongle.upload( files, target, uuid, service, channel )
            self.__rotate_dongle()
            logger.debug("upload in queue")
        except Exception, err:
            logger.error("Failed on upload")
            logger.exception(err)
            raise err

    def exposed_add_dongle(self, address, conns, name):
        '''
        This method gets called by the server during
        '''
        self.uploaders[address]=(conns, name)
        
    def exposed_stop(self):
        '''
        Exposed method to force upload stopping.
        (not used or valid at all!)
        '''
        logger.info("stop called")
        for k in self.__sequence:
            try:
                k.worker.stop()
                k.proxy.stop()
            except:
                pass

    def getDongle4Address(self, address):
        if not address:
            self.__index = self.__index or 0
            dongle = self.__sequence[self.__index]
        else:
            dongle = self.manager.getAdapter(address)
        return self.__dongles[dongle.bt_address]


    def exposed_start_pairing(self, address, dongle=None):
        try:
            logger.debug("pairing to address %s", address)
            uploader = self.getDongle4Address(dongle)
            uploader.pairDevice(address)
        except Exception, err:
            logger.exception(err)
            raise err

    def exposed_resolve_service(self, address, service='opp', dongle=None):
        dongle = self.getDongle4Address(dongle)
        dongle.resolveService(
            uuid=getattr(const.UUID, service, const.UUID['opp']), 
            address=address
            )


if __name__=='__main__':
    def listen(signal, **kwargs):
        print signal, kwargs

    import dbus
    import gobject
    import dbus.glib
    import dbus.mainloop.glib
    import threading
    import sys
    
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    
    manager=UploadManager(dbus.SystemBus())
    manager.remote_listener=listen
    manager.exposed_add_dongle('00:25:BF:01:00:9E',7, 'test')
    manager.exposed_refreshUploaders()
    
    gobject.threads_init()
    dbus.glib.init_threads()
    loop=gobject.MainLoop()
    manager.exposed_upload(((sys.argv[2], 1),), sys.argv[1], dongle_name='AIRcable')
    loop.run()




# -*- coding: utf-8 -*-
# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from preset import *

# keep a reference to the parser
__CONFIGGLUE_PARSER__ = parser

from net.aircable.openproximity.pluginsystem import pluginsystem
pluginsystem.find_plugins(locals()['OP_PLUGINS'])

import plug
for k in dir(plug):
    if not k.isupper():
        continue
    locals()[k].extend(getattr(plug, k))
    orig=parser.get('django', k.lower())
    orig.extend(getattr(plug, k))
    parser.set('django', k.lower(), orig)

if not os.access(locals()['STORE_PATH'], os.W_OK):
    locals()['STORE_PATH'] = os.path.expanduser("~/.openproximity")

# make sure we don't get loaded again!
sys.modules['django-web.settings'] = sys.modules[__name__]

def __get_match_dongle(options, address):
    def __parse(option, typ=int):
        if len(option.strip()) == 0:
            return None
        return typ(option)

    address = address.lower().strip()
    for rang, val, enable, name in options:
        rang = rang.lower().strip()

        if address.startswith(rang):
            out = { 
                'enable': __parse(enable, bool), 
                'value': __parse(val), 
                'name': __parse(name, str) 
            }
            return out

GETSCANNERDONGLE=partial(__get_match_dongle, locals()['OP_SCANNERS'])
GETUPLOADERDONGLE=partial(__get_match_dongle, locals()['OP_UPLOADERS'])

class InvalidVarException(object):
    def __mod__(self, missing):
        try:
            missing_str=unicode(missing)
        except:
            missing_str='Failed to create string representation'
            raise Exception('Unknown template variable %r %s' % (
                    missing, missing_str))
    def __contains__(self, search):
        if search=='%s':
            return True
        return False
TEMPLATE_STRING_IF_INVALID = InvalidVarException()



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from django.conf.urls.defaults import *
from django.views.generic import list_detail
from django import forms
from django.contrib.auth.decorators import login_required
import django.contrib.databrowse as databrowse
from django.contrib.databrowse.datastructures import EasyModel
from django.views.generic.simple import direct_to_template

from models import *
import views
import treeview

class RemoteRecordForm(forms.ModelForm):
    class Meta:
        model = DeviceRecord
    
databrowse_root = databrowse.DatabrowseSite()
databrowse_root.register(BluetoothDongle)
databrowse_root.register(ScannerBluetoothDongle)
databrowse_root.register(RemoteScannerBluetoothDongle)
databrowse_root.register(RemoteDevice)
databrowse_root.register(RemoteBluetoothDeviceFoundRecord)
databrowse_root.register(RemoteBluetoothDeviceFileTry)
databrowse_root.register(RemoteBluetoothDeviceFilesSuccess)
databrowse_root.register(RemoteBluetoothDeviceFilesRejected)
databrowse_root.register(MarketingCampaign)
databrowse_root.register(CampaignFile)

info_dict = {
    'form_class': RemoteRecordForm,
    'post_save_redirect': '../accepted/',
}

from microblog.sites import TweetSite
def convert(time_string):
    return datetime.strptime(time_string, '%Y-%m-%d %H:%M:%S')

blog = TweetSite(LogLine, 'time', convert)

urlpatterns = patterns('',
    (r'^accepted/$', views.add_record_accepted),
    (r'^genericadd/$', 'django.views.generic.create_update.create_object', info_dict),
    (r'^remoteadd/$', views.add_record),
    (r'^data/(.*)$', databrowse_root.root ),
    (r'^rpc/server/(?P<command>.+)', views.server_rpc_command),
    (r'^rpc/(?P<command>.+)', views.rpc_command),
    (r'^configure/dongle/(?P<address>.+)', views.configure_dongle),
    (r'^configure/dongle/', views.configure_dongle),
    (r'^configure/camp/(?P<camp_name>.+)', views.configure_campaign),
    (r'^configure/camp/', views.configure_campaign),
    (r'^stats/restart.*', views.stats_restart),
    (r'^file/grab/(?P<file>.+)', views.grab_file),
    (r'^microblog/', include(blog.urlpatterns)),
    (r'^last-seen/$', views.last_seen),
    (r'^treeview/', include(treeview.urlpatterns)),
    (r'^$', views.index),
)




# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):

        # Adding model 'LogLine'
        db.create_table('openproximity_logline', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('time', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('content', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('openproximity', ['LogLine'])

        # Adding model 'Setting'
        db.create_table('openproximity_setting', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=40)),
            ('value', self.gf('net.aircable.fields.PickledField')(max_length=200)),
        ))
        db.send_create_signal('openproximity', ['Setting'])

        # Adding model 'BluetoothDongle'
        db.create_table('openproximity_bluetoothdongle', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('address', self.gf('django.db.models.fields.CharField')(max_length=17)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
        ))
        db.send_create_signal('openproximity', ['BluetoothDongle'])

        # Adding model 'ScannerBluetoothDongle'
        db.create_table('openproximity_scannerbluetoothdongle', (
            ('bluetoothdongle_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.BluetoothDongle'], unique=True, primary_key=True)),
            ('priority', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('openproximity', ['ScannerBluetoothDongle'])

        # Adding model 'RemoteScannerBluetoothDongle'
        db.create_table('openproximity_remotescannerbluetoothdongle', (
            ('scannerbluetoothdongle_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.ScannerBluetoothDongle'], unique=True, primary_key=True)),
            ('local_dongle', self.gf('django.db.models.fields.related.ForeignKey')(related_name='remote_dongles', to=orm['openproximity.ScannerBluetoothDongle'])),
        ))
        db.send_create_signal('openproximity', ['RemoteScannerBluetoothDongle'])

        # Adding model 'UploaderBluetoothDongle'
        db.create_table('openproximity_uploaderbluetoothdongle', (
            ('bluetoothdongle_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.BluetoothDongle'], unique=True, primary_key=True)),
            ('max_conn', self.gf('django.db.models.fields.IntegerField')(default=7)),
        ))
        db.send_create_signal('openproximity', ['UploaderBluetoothDongle'])

        # Adding model 'MarketingCampaign'
        db.create_table('openproximity_marketingcampaign', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=True, blank=True)),
            ('name_filter', self.gf('django.db.models.fields.CharField')(max_length=30, null=True, blank=True)),
            ('addr_filter', self.gf('django.db.models.fields.CharField')(max_length=20, null=True, blank=True)),
            ('devclass_filter', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('start', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('end', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('dongle_name', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
            ('pin_code', self.gf('django.db.models.fields.CharField')(default='1234', max_length=16, null=True, blank=True)),
            ('fixed_channel', self.gf('django.db.models.fields.IntegerField')(default=None, null=True, blank=True)),
            ('service', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('rejected_count', self.gf('django.db.models.fields.IntegerField')(default=2)),
            ('rejected_timeout', self.gf('django.db.models.fields.IntegerField')(default=-1)),
            ('tries_count', self.gf('django.db.models.fields.IntegerField')(default=-1)),
            ('tries_timeout', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('accepted_count', self.gf('django.db.models.fields.IntegerField')(default=-1)),
            ('rssi_min', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('rssi_max', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal('openproximity', ['MarketingCampaign'])

        # Adding model 'CampaignFile'
        db.create_table('openproximity_campaignfile', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('chance', self.gf('django.db.models.fields.DecimalField')(default=1.0, null=True, max_digits=3, decimal_places=2, blank=True)),
            ('file', self.gf('django.db.models.fields.files.FileField')(max_length=100)),
            ('campaign', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['openproximity.MarketingCampaign'])),
        ))
        db.send_create_signal('openproximity', ['CampaignFile'])

        # Adding model 'RemoteDevice'
        db.create_table('openproximity_remotedevice', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('address', self.gf('django.db.models.fields.CharField')(max_length=17)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100, null=True, blank=True)),
            ('last_seen', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('devclass', self.gf('django.db.models.fields.IntegerField')(null=True)),
        ))
        db.send_create_signal('openproximity', ['RemoteDevice'])

        # Adding model 'DeviceRecord'
        db.create_table('openproximity_devicerecord', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('time', self.gf('django.db.models.fields.DateTimeField')()),
            ('dongle', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['openproximity.BluetoothDongle'], null=True, blank=True)),
        ))
        db.send_create_signal('openproximity', ['DeviceRecord'])

        # Adding model 'RemoteBluetoothDeviceRecord'
        db.create_table('openproximity_remotebluetoothdevicerecord', (
            ('devicerecord_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.DeviceRecord'], unique=True, primary_key=True)),
            ('remote', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['openproximity.RemoteDevice'])),
        ))
        db.send_create_signal('openproximity', ['RemoteBluetoothDeviceRecord'])

        # Adding model 'RemoteBluetoothDeviceFoundRecord'
        db.create_table('openproximity_remotebluetoothdevicefoundrecord', (
            ('remotebluetoothdevicerecord_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.RemoteBluetoothDeviceRecord'], unique=True, primary_key=True)),
            ('_RemoteBluetoothDeviceFoundRecord__rssi', self.gf('django.db.models.fields.CommaSeparatedIntegerField')(max_length=200)),
        ))
        db.send_create_signal('openproximity', ['RemoteBluetoothDeviceFoundRecord'])

        # Adding model 'RemoteBluetoothDeviceSDP'
        db.create_table('openproximity_remotebluetoothdevicesdp', (
            ('remotebluetoothdevicerecord_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.RemoteBluetoothDeviceRecord'], unique=True, primary_key=True)),
            ('channel', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('openproximity', ['RemoteBluetoothDeviceSDP'])

        # Adding model 'RemoteBluetoothDeviceNoSDP'
        db.create_table('openproximity_remotebluetoothdevicenosdp', (
            ('remotebluetoothdevicerecord_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.RemoteBluetoothDeviceRecord'], unique=True, primary_key=True)),
        ))
        db.send_create_signal('openproximity', ['RemoteBluetoothDeviceNoSDP'])

        # Adding model 'RemoteBluetoothDeviceSDPTimeout'
        db.create_table('openproximity_remotebluetoothdevicesdptimeout', (
            ('remotebluetoothdevicerecord_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.RemoteBluetoothDeviceRecord'], unique=True, primary_key=True)),
        ))
        db.send_create_signal('openproximity', ['RemoteBluetoothDeviceSDPTimeout'])

        # Adding model 'RemoteBluetoothDeviceFileTry'
        db.create_table('openproximity_remotebluetoothdevicefiletry', (
            ('remotebluetoothdevicerecord_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.RemoteBluetoothDeviceRecord'], unique=True, primary_key=True)),
            ('campaign', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['openproximity.MarketingCampaign'])),
        ))
        db.send_create_signal('openproximity', ['RemoteBluetoothDeviceFileTry'])

        # Adding model 'RemoteBluetoothDeviceFilesRejected'
        db.create_table('openproximity_remotebluetoothdevicefilesrejected', (
            ('remotebluetoothdevicefiletry_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.RemoteBluetoothDeviceFileTry'], unique=True, primary_key=True)),
            ('ret_value', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('openproximity', ['RemoteBluetoothDeviceFilesRejected'])

        # Adding model 'RemoteBluetoothDeviceFilesSuccess'
        db.create_table('openproximity_remotebluetoothdevicefilessuccess', (
            ('remotebluetoothdevicefiletry_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['openproximity.RemoteBluetoothDeviceFileTry'], unique=True, primary_key=True)),
        ))
        db.send_create_signal('openproximity', ['RemoteBluetoothDeviceFilesSuccess'])


    def backwards(self, orm):
        
        # Deleting model 'UserProfile'
        db.delete_table('openproximity_userprofile')

        # Deleting model 'LogLine'
        db.delete_table('openproximity_logline')

        # Deleting model 'Setting'
        db.delete_table('openproximity_setting')

        # Deleting model 'BluetoothDongle'
        db.delete_table('openproximity_bluetoothdongle')

        # Deleting model 'ScannerBluetoothDongle'
        db.delete_table('openproximity_scannerbluetoothdongle')

        # Deleting model 'RemoteScannerBluetoothDongle'
        db.delete_table('openproximity_remotescannerbluetoothdongle')

        # Deleting model 'UploaderBluetoothDongle'
        db.delete_table('openproximity_uploaderbluetoothdongle')

        # Deleting model 'MarketingCampaign'
        db.delete_table('openproximity_marketingcampaign')

        # Deleting model 'CampaignFile'
        db.delete_table('openproximity_campaignfile')

        # Deleting model 'RemoteDevice'
        db.delete_table('openproximity_remotedevice')

        # Deleting model 'DeviceRecord'
        db.delete_table('openproximity_devicerecord')

        # Deleting model 'RemoteBluetoothDeviceRecord'
        db.delete_table('openproximity_remotebluetoothdevicerecord')

        # Deleting model 'RemoteBluetoothDeviceFoundRecord'
        db.delete_table('openproximity_remotebluetoothdevicefoundrecord')

        # Deleting model 'RemoteBluetoothDeviceSDP'
        db.delete_table('openproximity_remotebluetoothdevicesdp')

        # Deleting model 'RemoteBluetoothDeviceNoSDP'
        db.delete_table('openproximity_remotebluetoothdevicenosdp')

        # Deleting model 'RemoteBluetoothDeviceSDPTimeout'
        db.delete_table('openproximity_remotebluetoothdevicesdptimeout')

        # Deleting model 'RemoteBluetoothDeviceFileTry'
        db.delete_table('openproximity_remotebluetoothdevicefiletry')

        # Deleting model 'RemoteBluetoothDeviceFilesRejected'
        db.delete_table('openproximity_remotebluetoothdevicefilesrejected')

        # Deleting model 'RemoteBluetoothDeviceFilesSuccess'
        db.delete_table('openproximity_remotebluetoothdevicefilessuccess')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'openproximity.bluetoothdongle': {
            'Meta': {'object_name': 'BluetoothDongle'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '17'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'})
        },
        'openproximity.campaignfile': {
            'Meta': {'object_name': 'CampaignFile'},
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['openproximity.MarketingCampaign']"}),
            'chance': ('django.db.models.fields.DecimalField', [], {'default': '1.0', 'null': 'True', 'max_digits': '3', 'decimal_places': '2', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'openproximity.devicerecord': {
            'Meta': {'object_name': 'DeviceRecord'},
            'dongle': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['openproximity.BluetoothDongle']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time': ('django.db.models.fields.DateTimeField', [], {})
        },
        'openproximity.logline': {
            'Meta': {'object_name': 'LogLine'},
            'content': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'openproximity.marketingcampaign': {
            'Meta': {'object_name': 'MarketingCampaign'},
            'accepted_count': ('django.db.models.fields.IntegerField', [], {'default': '-1'}),
            'addr_filter': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'devclass_filter': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'dongle_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'fixed_channel': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name_filter': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'pin_code': ('django.db.models.fields.CharField', [], {'default': "'1234'", 'max_length': '16', 'null': 'True', 'blank': 'True'}),
            'rejected_count': ('django.db.models.fields.IntegerField', [], {'default': '2'}),
            'rejected_timeout': ('django.db.models.fields.IntegerField', [], {'default': '-1'}),
            'rssi_max': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rssi_min': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'service': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'tries_count': ('django.db.models.fields.IntegerField', [], {'default': '-1'}),
            'tries_timeout': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'openproximity.remotebluetoothdevicefilesrejected': {
            'Meta': {'object_name': 'RemoteBluetoothDeviceFilesRejected', '_ormbases': ['openproximity.RemoteBluetoothDeviceFileTry']},
            'remotebluetoothdevicefiletry_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceFileTry']", 'unique': 'True', 'primary_key': 'True'}),
            'ret_value': ('django.db.models.fields.IntegerField', [], {})
        },
        'openproximity.remotebluetoothdevicefilessuccess': {
            'Meta': {'object_name': 'RemoteBluetoothDeviceFilesSuccess', '_ormbases': ['openproximity.RemoteBluetoothDeviceFileTry']},
            'remotebluetoothdevicefiletry_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceFileTry']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicefiletry': {
            'Meta': {'object_name': 'RemoteBluetoothDeviceFileTry', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['openproximity.MarketingCampaign']"}),
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicefoundrecord': {
            'Meta': {'object_name': 'RemoteBluetoothDeviceFoundRecord', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            '_RemoteBluetoothDeviceFoundRecord__rssi': ('django.db.models.fields.CommaSeparatedIntegerField', [], {'max_length': '200'}),
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicenosdp': {
            'Meta': {'object_name': 'RemoteBluetoothDeviceNoSDP', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicerecord': {
            'Meta': {'object_name': 'RemoteBluetoothDeviceRecord', '_ormbases': ['openproximity.DeviceRecord']},
            'devicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.DeviceRecord']", 'unique': 'True', 'primary_key': 'True'}),
            'remote': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['openproximity.RemoteDevice']"})
        },
        'openproximity.remotebluetoothdevicesdp': {
            'Meta': {'object_name': 'RemoteBluetoothDeviceSDP', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'channel': ('django.db.models.fields.IntegerField', [], {}),
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicesdptimeout': {
            'Meta': {'object_name': 'RemoteBluetoothDeviceSDPTimeout', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotedevice': {
            'Meta': {'object_name': 'RemoteDevice'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '17'}),
            'devclass': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_seen': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'openproximity.remotescannerbluetoothdongle': {
            'Meta': {'object_name': 'RemoteScannerBluetoothDongle', '_ormbases': ['openproximity.ScannerBluetoothDongle']},
            'local_dongle': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'remote_dongles'", 'to': "orm['openproximity.ScannerBluetoothDongle']"}),
            'scannerbluetoothdongle_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.ScannerBluetoothDongle']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.scannerbluetoothdongle': {
            'Meta': {'object_name': 'ScannerBluetoothDongle', '_ormbases': ['openproximity.BluetoothDongle']},
            'bluetoothdongle_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.BluetoothDongle']", 'unique': 'True', 'primary_key': 'True'}),
            'priority': ('django.db.models.fields.IntegerField', [], {})
        },
        'openproximity.setting': {
            'Meta': {'object_name': 'Setting'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'value': ('net.aircable.fields.PickledField', [], {'max_length': '200'})
        },
        'openproximity.uploaderbluetoothdongle': {
            'Meta': {'object_name': 'UploaderBluetoothDongle', '_ormbases': ['openproximity.BluetoothDongle']},
            'bluetoothdongle_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.BluetoothDongle']", 'unique': 'True', 'primary_key': 'True'}),
            'max_conn': ('django.db.models.fields.IntegerField', [], {'default': '7'})
        },
    }

    complete_apps = ['openproximity']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import threading, time, traceback, sys, os
from openproximity.rpc import server
from rpyc import Service, async
from rpyc.utils.server import ThreadedServer, ForkingServer
from django.core.management.base import BaseCommand
from django.conf import settings
from optparse import make_option, OptionParser
from configglue.glue import schemaconfigglue


SERVER_OPTIONS = {
    'host': settings.RPC_HOST,
    'port': settings.RPC_PORT,
    'autoregister': settings.RPC_AUTOREGISTER,
    'threaded': settings.RPC_THREADED,
    'daemonize': settings.RPC_DAEMONIZE,
    'pidfile': settings.RPC_PIDFILE,
    'server_user': settings.RPC_SERVER_USER,
    'server_group': settings.RPC_SERVER_GROUP,
}

class Command(BaseCommand):
    help = "RPyC Server for OpenProximity."

    def handle(self, *args, **options):
        from django.conf import settings
        from django.utils import translation
        from net.aircable import utils
        utils.log_setup(settings, True)
        runserver(args)

    def usage(self, subcommand):
        return self.help

    def create_parser(self, prog_name, subcommand):
        """
        Add all our SchemaConfigParser's options so they can be shown
        in help messages and such.
        """
        parser = OptionParser(prog=prog_name,
            usage=self.usage(subcommand),
            version=self.get_version(),
            option_list=self.option_list)

        configglue_parser = settings.__CONFIGGLUE_PARSER__
        configglue_parser.setmode("rpyc")
        op, options, args = schemaconfigglue(configglue_parser, op=parser)
        return op

def change_uid_gid(uid, gid=None):
    """Try to change UID and GID to the provided values.
    UID and GID are given as names like 'nobody' not integer.

    Src: http://mail.mems-exchange.org/durusmail/quixote-users/4940/1/
    """
    if not os.geteuid() == 0:
        # Do not try to change the gid/uid if not root.
        return
    (uid, gid) = get_uid_gid(uid, gid)
    os.setgid(gid)
    os.setuid(uid)

def get_uid_gid(uid, gid=None):
    """Try to change UID and GID to the provided values.
    UID and GID are given as names like 'nobody' not integer.

    Src: http://mail.mems-exchange.org/durusmail/quixote-users/4940/1/
    """
    import pwd, grp
    uid, default_grp = pwd.getpwnam(uid)[2:4]
    if gid is None:
        gid = default_grp
    else:
        try:
            gid = grp.getgrnam(gid)[2]
        except KeyError:
            gid = default_grp
    return (uid, gid)

def start_server(options):
    """
    Start RPyC server
    """

    if options['daemonize'] and options['server_user'] and \
            options['server_group']:
        #ensure the that the daemon runs as specified user
        change_uid_gid(options['server_user'], options['server_group'])

    if options['threaded']:
        serv = ThreadedServer(server.OpenProximityService, options['host'],
            port = options['port'], auto_register = options['autoregister'])
    else:
        print "Forking server is not a tested server!"
        serv = ForkingServer(server.OpenProximityService, options['host'],
            port = options['port'], auto_register = options['autoregister'])

    try:
        serv.start()
    except KeyboardInterrupt, err:
        serv.stop()
    
    
def runserver(argset=[], **kwargs):
    # Get the options
    options = SERVER_OPTIONS.copy()
    options.update(kwargs)

    from net.aircable.utils import getLogger
    logger = getLogger(__name__)

    if options['daemonize']:
        if not options['pidfile']:
            options['pidfile'] = '/var/run/openproximity_rpc_%s.pid' % \
                options['port']

        logger.info("storing pid file in %s" % options['pidfile'])
        from django.utils.daemonize import become_daemon
        
        logger.info("daemonizing")
        become_daemon()
        logger.info("daemonized")

        fp = open(options['pidfile'], 'w')
        fp.write("%d\n" % os.getpid())
        fp.close()

    # Start the rpc server
    logger.debug('starting server with options %s' % options)
    start_server(options)
    
if __name__ == '__main__':
    runserver(sys.argv[1:])



#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from django import template

register = template.Library()

@register.inclusion_tag("MochiKit.js")
def MochiKit():
    return {}

@register.inclusion_tag("log.html")
def Tweet(element, 
        field='pk', 
        url='./', 
        line_format='post.pk + " " + post.fields', 
        interval=5000, 
        amount=100, 
        first_amount=5):

    return {
	    'element': element,
	    'field': field,
	    'url': url,
	    'amount': amount,
	    'interval': interval,
	    'line_format': line_format,
	    'first_amount': first_amount,
	}




# -*- coding: utf-8 -*-
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os, sys
from net.aircable.openproximity.pluginsystem import pluginsystem

from net.aircable.utils import getLogger
logger = getLogger(__name__)

TEMPLATE_DIRS=()
LOCALE_PATHS=()
INSTALLED_APPS=()

logger.info("starting up plugins")
for plugin in pluginsystem.get_plugins('django'):
    if plugin.provides.get('TEMPLATE_DIRS', None) is not None :
        TEMPLATE_DIRS += ( 
            os.path.join(plugin.path, plugin.provides['TEMPLATE_DIRS']), )
    if plugin.provides.get('LOCALE_PATHS', None) is not None:
        LOCALE_PATHS+=( 
            os.path.join(plugin.path, plugin.provides['LOCALE_PATHS']), )
    if plugin.provides.get('django', False):
        INSTALLED_APPS+=( plugin.name, )

logger.info("starting up plugin providers")
for plugin in pluginsystem.get_plugins('plugin_provider'):
    for plug in pluginsystem.get_plugins(
            plugin.provides['plugin_provider_name']):
        INSTALLED_APPS+=( plug.name, )

logger.info("plugin system initied")
logger.info("settings.py loaded")



# -*- coding: utf-8 -*-
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from models import *
from django.contrib import admin
from django.utils.translation import ugettext as _
from timezones.utils import adjust_datetime_to_timezone
from timezones.zones import tz as TZ
from timezones.zones import PRETTY_TIMEZONE_CHOICES


class SettingAdmin(admin.ModelAdmin):
    list_display = ('name', 'value')

admin.site.register(Setting, SettingAdmin)

class DongleAdmin(admin.ModelAdmin):
    list_display = ( 'address', 'name', 'enabled' )

class RemoteDongleAdmin(admin.ModelAdmin):
    list_display = ( 'address', 'name', 'local_dongle', 'priority', 'enabled' )

class RemoteDongleInline(admin.TabularInline):
    model = RemoteScannerBluetoothDongle
    fk_name = 'local_dongle'
    fields = ('address', 'name','priority', 'enabled')
    extra = 7
    template = 'op/tabular_remotescanner.html'
        
class ScannerDongleAdmin(admin.ModelAdmin):
    inlines = [ RemoteDongleInline ]
    list_display = ( 'address', 'name', 'priority', 'enabled' )

admin.site.register(ScannerBluetoothDongle, ScannerDongleAdmin)
admin.site.register(RemoteScannerBluetoothDongle, RemoteDongleAdmin)
admin.site.register(UploaderBluetoothDongle, DongleAdmin)

class CampaignFileAdmin(admin.StackedInline):
    model = CampaignFile
    date_hierarchy = None


class MarketingCampaignAdmin(admin.ModelAdmin):
    class Media:
        js = (
            "/static/js/jquery-1.6.2.min.js",
            "/static/js/Base.js",
            "/static/js/DateTime.js",
            "/static/js/date.js",
        )

    fieldsets = (
      (None, {
        'fields': ('name', 'enabled', 'service', 'pin_code'),
      }),
      ('Times filter',{
        'fields': ('rejected_count', 'rejected_timeout', 
            'tries_count', 'tries_timeout',
            'accepted_count')
      }),
      ('RSSI filter', {
        'fields': ('rssi_min', 'rssi_max'),
      }), 
      ('Dongles settings',{
        'fields': ('dongle_name',),
      }),
      ('Timing Filters', {
        'classes': ('collapse', ),
        'fields': ('start', 'end')
      }),
      ('Extra Filter Settings', {
        'classes': ('collapse', ),
        'fields': ('name_filter', 'addr_filter', 'devclass_filter')
      }),
      ('Expert Settings', {
        'classes': ('collapse', ),
        'fields': ('fixed_channel', 'concurrent_scanning', 'upload_on_discovered')
      }),
    )

    inlines = [ CampaignFileAdmin, ]

    list_display = ( 'name', 
        'service',
        'pin_code',
        'format_start',
        'format_end',
        'rssi_min',
        'rssi_max',
        'name_filter', 
        'addr_filter', 
        'devclass_filter',
        'enabled'
    )
    list_filter = ( 'service', 
        'start', 
        'end', 
        'rssi_min',
        'rssi_max',
        'name_filter', 
        'addr_filter',
        'devclass_filter',
        'enabled'
    )

    ordering = [ 'name', 'service', 'start', 'end', 'rssi_min', 'rssi_max' ]

    def save_model(self, request, obj, form, change):
        utz = request.user.get_profile().timezone
        if obj.start:
            obj.start = adjust_datetime_to_timezone(obj.start, utz, TZ)
        if obj.end:
            obj.end = adjust_datetime_to_timezone(obj.end, utz, TZ)
        obj.save()

    def change_view(self, request, object_id, extra_context=None):
        zone = request.user.get_profile().timezone.zone
        time_zone = dict(PRETTY_TIMEZONE_CHOICES)[zone][:10]
        my_context = {'time_zone': time_zone}
        return super(MarketingCampaignAdmin, self).\
            change_view(request,object_id,my_context)

    def changelist_view(self, request, extra_context=dict()):
        zone = request.user.get_profile().timezone.zone
        time_zone = dict(PRETTY_TIMEZONE_CHOICES)[zone][:10]
        extra_context['time_zone']=time_zone
        return super(MarketingCampaignAdmin, self).\
            changelist_view(request,extra_context)

    def __format(self, obj):
        return '<div class="datetimezulu">'+\
            obj.strftime("%Y-%m-%dT%H:%M:%SZ")+\
        '</div>'

    def format_start(self, obj):
        return self.__format(obj.start)
    format_start.short_description = _('Start')
    format_start.allow_tags=True

    def format_end(self, obj):
        return self.__format(obj.end)
    format_end.short_description = _('End')
    format_end.allow_tags=True


admin.site.register(MarketingCampaign, MarketingCampaignAdmin)



#!/usr/bin/env python
#    OpenProximity2.0 is a proximity marketing OpenSource system.
#    Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import sys, os, fnmatch

DJANGO_PATH=os.path.dirname(os.path.realpath(__file__))
OP_PATH=os.path.dirname(DJANGO_PATH)

paths = list()
paths.append(os.path.join("usr", "lib", "openproximity"))
paths.append(os.path.expanduser(os.path.join('~', '.openproximity')))

def find_all_eggs(parent):
  for par, subdirs, files in os.walk(parent, followlinks=True):
    for f in fnmatch.filter(files, '*.egg'):
      yield os.path.join(par, par, f)

for p in list(paths):
  paths.extend(find_all_eggs(p))

sys.path = paths + sys.path

try:
    from openproximity import __version__ as version
except:
    version = 'ND'
os.environ['OP_VERSION'] = version




# -*- coding: utf-8 ; Mode: python; tab-width: 4 ; indent-tabs-mode: nil -*-
# OpenProximity2.0 is a proximity marketing OpenSource system.
# Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from net.aircable.openproximity.signals import scanner as signals
from openproximity.models import *
from random import random
from re import compile
from rpyc import async
from rpyc.lib.compat import ByValWrapper
from net.aircable.openproximity.pluginsystem import pluginsystem
from net.aircable.utils import trace
import traceback, sys

from net.aircable.utils import getLogger
logger = getLogger(__name__)

def is_known_dongle(address, klass):
    return klass.objects.filter(address=address).count() > 0
    
AIRCABLE_MAC=['00:50:C2', '00:25:BF']
ADDRESS_MAC=compile("([0-9A-F]{2}\:){5}([0-9A-F]{2})")

def isAIRcable(address):
    return address[:8].upper() in AIRCABLE_MAC

def do_upload(uploader, 
                    files, 
                    remote, 
                    service='opp', 
                    dongle_name=None, 
                    channel=None,
                    dongle=None):
    logger.info("do_upload")
    logger.debug("About to call upload")

    
    if not channel:
        logger.info("asking for sdp resolving")
        uploader.resolve_service( remote, service, 
                                  dongle.address if dongle else None )
    elif not RemoteBluetoothDevicePairing.isPaired(remote):
        uploader.start_pairing(remote, 
                               dongle=dongle.address if dongle else None)
    else:
        dongles = RemoteBluetoothDevicePairing.getDonglesForRemote(remote)
        if dongle and dongle.address in dongles:
            dongle = dongle.address
        else:
            dongle = dongles[-1] # use last paired device
        uploader.upload(ByValWrapper(files), 
                        remote, 
                        service, 
                        dongle_name=dongle_name, 
                        channel=channel,
                        uploader=str(dongle))
        logger.debug("upload called async")
    
def get_files_from_campaign(camp, record):
    if camp.hasAccepted(record.remote):
        logger.info("Allready accepted")
        raise StopIteration
        
    c = camp.getRejectedCount(record.remote)
    d = camp.getTimeoutCount(record.remote)

    if c>0 or d>0:
        logger.debug("All ready rejected %s times, and timedout %s" % (c, d))
        try_ = camp.tryAgain(remote=record.remote, record=None)
        logger.info("try again: %s" % try_)
        if not try_:
            raise StopIteration

    files__ = camp.campaignfile_set
    files__ = files__.filter(chance__isnull=True) | files__.filter(
                                        chance__gte=str(random())
    )
    for f in files__:
        logger.debug('going to upload %s' % f.file)
        yield str(f.file.name), camp.pk

def found_action_upload(remote, record = None, line=None):
    from openproximity.rpc.server import OpenProximityService
    
    uploader = OpenProximityService.getUploader()

    if uploader is None:
        line.content+=" no uploaders, can't handle"
        line.save()
        return True

    logger.info("found uploader")

    camps = getMatchingCampaigns(
            remote, 
            enabled=True, 
            record=record, 
            classes=[MarketingCampaign,]
    )


    if len(camps)==0:
        line.content+=" no matching campaings, not handling"
        line.save()
        logger.info("no campaigns")
        return True

    channel = None

    files=list()
    name=None
    service='opp'
    use_same = False

    for camp in camps:
        files.extend( list(get_files_from_campaign(camp, record)) )
        if camp.dongle_name:
            name = camp.dongle_name
        service = camp.get_service_display()
        if camp.fixed_channel and camp.fixed_channel > -1:
          channel = camp.fixed_channel
        if camp.upload_on_discovered:
            use_same = True

    sdp = RemoteBluetoothDeviceSDP.objects.filter(remote__pk=record.remote.pk)
    if sdp.count() > 0:
        channel = sdp.order_by("-time")[0].channel
        line.content += "using previous resolved channel %i" % channel

    logger.info("going to upload %s files" % len(files))
    if len(files) > 0:
        OpenProximityService.addPending(record.remote.address, uploader)
        do_upload(uploader, files, record.remote.address, service, name, 
                channel=channel, dongle=dongle if use_same else None)
        line.content+=" uploading files"
    else:
        line.content+=" no files to upload"
    
    line.save()

def found_action(address, record, dongle):
    line = LogLine()
    line.content="Found action for: %s" % address
    try:
        for plugin in pluginsystem.get_plugins('found_action'):
            logger.info("found action trying with %s" % plugin.name)
            service = plugin.rpc['found_action'](record=record, dongle=dongle)
            if service:
                logger.info("plugin has handled")
                line.content+=" %s is handling" % getattr(plugin, 'name', 
                                                                    'plugin')
                line.save()
                OpenProximityService.addPending(address, service)
                return True
    except Exception, err:
        logger.error("plugin do_action")
        logger.exception(err)

    return found_action_upload(record.remote, record, line)

def schedule_try_upload(klass, remote_address, rule_id, record_id):
    line = LogLine()
    camp = klass.objects.get(pk=rule_id)
    remote = RemoteDevice.objects.get(address=remote_address)
    record = RemoteBluetoothDeviceFilesRejected.objects.get(pk=record_id)
    line.content = "Scheduled event for %s" % remote_address
    logger.info("scheduled try for %s %s" % (remote, camp))
    found_action_upload(remote, record, line)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#   OpenProximity2.0 is a proximity marketing OpenSource system.
#   Copyright (C) 2010,2009,2008 Naranjo Manuel Francisco <manuel@aircable.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation version 2 of the License.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import gc
from net.aircable.utils import getLogger
logger = getLogger(__name__)
from net.aircable.openproximity.pluginsystem import pluginsystem
import net.aircable.openproximity.signals as signals
import scanner, uploader
import threading, time, traceback, sys

from django.db import transaction, close_connection, reset_queries
from openproximity.models import CampaignFile, getMatchingCampaigns, \
    RemoteDevice
from rpyc import Service, async
from rpyc.utils.server import ThreadedServer, ForkingServer
from dispatcher import Dispatcher

enabled = True # useful when tables are been drop
all_dongles = set()

def db_ready():
    try:
        from openproximity.models import BluetoothDongle
        BluetoothDongle.objects.count()
        return True
    except Exception, err:
        logger.info("Database not ready")
        logger.exception(err)
        return False

class OpenProximityService(Service):
    dongles = None
    remote_quit = None

    def __init__(self, *args, **kwargs):
        logger.info("New OpenProximityServer")
        Service.__init__(self, *args, **kwargs)
        OpenProximityService.instances.append(self)

    def on_connect(self):
        OpenProximityService.services.add(self)

    def on_disconnect(self):
        a = [ p for p in OpenProximityService.pending \
            if OpenProximityService.pending[p]==self]
        if len(a) > 0:
            logger.info(
                "a client disconnected, clearing %s pending transactions" % 
                len(a))
        for p in a:
               OpenProximityService.pending.pop(p)
        OpenProximityService.instances.remove(self)
        OpenProximityService.services.remove(self)
        del self

    def exit(self, exit):
        for ser in OpenProximityService.services:
            if ser.remote_quit is not None:
                try:
                    ser.remote_quit()
                except:
                    pass
        OpenProximityService.pending = dict()
        if exit:
            sys.exit(3) # restart me please

    @transaction.commit_manually
    def exposed_listener(self, signal, *args, **kwargs):
        global enabled

        if not enabled:
            logger.debug("rpc is locked, dropping signal %s" % signal)
            return

        logger.debug("exposed_listener %s %s %s" % ( signal, args, kwargs) )

        try:
            for plugin in pluginsystem.get_plugins('rpc'):
                plugin.rpc['handle'](
                    signal=signal, 
                    *args, **kwargs)
            transaction.commit() 
            # commit only after all the plugins have handled
        except Exception, err:
            logger.error("rpc listener while doing plugins")
            logger.exception(err)
            transaction.rollback() # oops rollback

        try:
            if signals.isScannerSignal(signal):
                scanner.handle(signal=signal, *args, **kwargs)
            elif signals.isUploaderSignal(signal):
                uploader.handle(signal=signal, *args, **kwargs)
            transaction.commit()
            # commit only after scanner and upload has done it's work
        except Exception, err:
            logger.error("rpc listener while doing scanner or uploader")
            logger.exception(err)
            transaction.rollback() # oops rollback
        logger.debug("freeing up some memory: %s" % len(gc.garbage))

        #this things happen when a web request is completed
        #we don't have a web request, have to do it our own
        reset_queries()
        close_connection()
        del gc.garbage[:]
        gc.collect()
        logger.debug("done: %s" % len(gc.garbage))

    def exposed_generic_register(self, remote_quit=None, 
            dongles=None, client=None):
        logger.info("generic register")

        all_dongles.update(dongles)

        if not enabled:
            return

        if not db_ready():
            return

        try:
            for plugin in pluginsystem.get_plugins('rpc_register'):
                logger.debug("plugin %s provides rpc register" % 
                    plugin.name)
                if plugin.rpc['register'](dongles=dongles, client=client):
                    # wrap all calls as async, to avoid collitions
                    self.remote_quit = async(remote_quit)
                    logger.info("plugin %s handled rpc_register" % 
                        plugin.name)
                    return
        except Exception, err:
            logger.exception(err)

    def exposed_scanner_register(self,  client = None,
            remote_quit=None, dongles=None):
        global enabled
        all_dongles.update(dongles)
        self.dongles = set()
        # wrap all calls as async, to avoid collitions
        self.add_dongle = async(client.add_dongle)
        self.scanner = client
        self.setConcurrent = async(client.setConcurrent)
        self.refreshScanners = async(client.refreshScanners)
        self.doScan = async(client.doScan)
        self.startScanningCycle = async(client.startScanningCycle)
        self.remote_quit = async(remote_quit)
        self.noCampaigns = async(client.noCampaigns)

        if not enabled:
            return

        if not db_ready():
            return

        logger.info("scanner register %s" % dongles)
        for dongle in dongles:
            self.dongles.add( str(dongle), )

        for dongle, priority, name in scanner.get_dongles(dongles): # local
            logger.info("%s: %s [%s]" % (dongle, name, priority) )
            self.add_dongle(dongle, priority, name)
        self.refreshScanners()

    def exposed_uploader_register(self, client = None,
            remote_quit=None, dongles=None):
        global enabled
        all_dongles.update(dongles)
        self.dongles = set()
        self.add_dongle = async(client.add_dongle)
        self.upload = async(client.upload)
        self.resolve_service = async(client.resolve_service)
        self.start_pairing = async(client.start_pairing)
        self.remote_quit = async(remote_quit)
        self.refreshUploaders = async(client.refreshUploaders)

        self.uploader = client

        if not enabled:
            return

        if not db_ready():
            return

        logger.info("uploader register")
        for dongle in dongles:
            self.dongles.add( str(dongle), )

        for dongle, max_conn, name in uploader.get_dongles(dongles):
            logger.info("%s: %s[%s]" % (dongle, name, max_conn))
            self.add_dongle(dongle, max_conn, name)
        self.refreshUploaders()

    def exposed_getFile(self, path):
        logger.info("getFile %s" % path)
        return CampaignFile.objects.get(file=path).file.read()

    def exposed_getUploadersCount(self):
        count = 0
        for ser in OpenProximityService.services:
            if getattr(ser,'uploader',None) is not None:
                count += 1
        return count

    def exposed_getScannersCount(self):
        count = 0
        for ser in OpenProximityService.services:
            if getattr(ser,'scanner',None) is not None:
                count += 1
        return count

    def exposed_getDongles(self):
        def getDongles_internal(self):
            for ser in OpenProximityService.services:
                if ser.dongles is not None:
                    for d in ser.dongles:
                        yield d

        return set(getDongles_internal(self))
    
    @classmethod
    def getDongleForService(klass, attribute):
        for service in klass.instances:
            if attribute in dir(service):
                return service
        return None

    @classmethod
    def getUploader(klass):
        return klass.getDongleForService('uploader')
    
    @classmethod
    def getScanner(klass):
        return klass.getDongleForService('scanner')
    
    @classmethod
    def getPendings(klass):
        return OpenProximityService.pending
    
    @classmethod
    def isPending(klass, address):
        return address in OpenProximityService.pending
    
    @classmethod
    def removePending(klass, address):
        return OpenProximityService.pending.pop(address)
    
    @classmethod
    def addPending(klass, address, service):
        OpenProximityService.pending[address]=service

    def __str__(self):
        return str(dir(self))

    def exposed_exit(self):
        logger.info("remoteExit")
        return self.exit(True)

    def exposed_restart(self):
        logger.info("remoteRestart")
        return self.exit(False)

    def exposed_Lock(self):
        logger.info("lock requested")
        global enabled
        enabled = False
        self.exit(False)

    def exposed_Unlock(self):
        logger.info("lock released")
        global enabled
        enabled = True
        self.exit(False)

    def exposed_getPIN(self, remote, local):
        logger.info("getPIN request for %s->%s" % (local, remote) )
        remote = RemoteDevice.getRemoteDevice(address=remote)
        try:
            for camp in getMatchingCampaigns(remote=remote, enabled=True):
                if not camp.pin_code:
                    continue
                logger.debug("pin code: %s" % camp.pin_code)
                return camp.pin_code
        except Exception, err:
            logger.error(err)
            logger.exception(err)
        logger.info("No pin code")

        raise Exception("No pin code found")

OpenProximityService.instances = []
OpenProximityService.services = set()
OpenProximityService.pending = dict()



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'RemoteBluetoothDevicePairing.msg'
        db.add_column('openproximity_remotebluetoothdevicepairing', 'msg', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True), keep_default=False)

        # Adding field 'RemoteBluetoothDevicePairing.exception'
        db.add_column('openproximity_remotebluetoothdevicepairing', 'exception', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True), keep_default=False)


    def backwards(self, orm):
        
        # Deleting field 'RemoteBluetoothDevicePairing.msg'
        db.delete_column('openproximity_remotebluetoothdevicepairing', 'msg')

        # Deleting field 'RemoteBluetoothDevicePairing.exception'
        db.delete_column('openproximity_remotebluetoothdevicepairing', 'exception')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'openproximity.bluetoothdongle': {
            'Meta': {'object_name': 'BluetoothDongle'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '17'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'})
        },
        'openproximity.campaignfile': {
            'Meta': {'object_name': 'CampaignFile'},
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['openproximity.MarketingCampaign']"}),
            'chance': ('django.db.models.fields.DecimalField', [], {'default': '1.0', 'null': 'True', 'max_digits': '3', 'decimal_places': '2', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'openproximity.devicerecord': {
            'Meta': {'ordering': "['time']", 'object_name': 'DeviceRecord'},
            'dongle': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['openproximity.BluetoothDongle']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time': ('django.db.models.fields.DateTimeField', [], {})
        },
        'openproximity.logline': {
            'Meta': {'object_name': 'LogLine'},
            'content': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'time': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'openproximity.marketingcampaign': {
            'Meta': {'ordering': "['start', 'end', 'name']", 'object_name': 'MarketingCampaign'},
            'accepted_count': ('django.db.models.fields.IntegerField', [], {'default': '-1'}),
            'addr_filter': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'concurrent_scanning': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'devclass_filter': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'dongle_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'fixed_channel': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name_filter': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'pin_code': ('django.db.models.fields.CharField', [], {'default': "'1234'", 'max_length': '16', 'null': 'True', 'blank': 'True'}),
            'rejected_count': ('django.db.models.fields.IntegerField', [], {'default': '2'}),
            'rejected_timeout': ('django.db.models.fields.IntegerField', [], {'default': '-1'}),
            'rssi_max': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rssi_min': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'service': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'start': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'tries_count': ('django.db.models.fields.IntegerField', [], {'default': '-1'}),
            'tries_timeout': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'upload_on_discovered': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'openproximity.remotebluetoothdevicefilesrejected': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDeviceFilesRejected', '_ormbases': ['openproximity.RemoteBluetoothDeviceFileTry']},
            'remotebluetoothdevicefiletry_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceFileTry']", 'unique': 'True', 'primary_key': 'True'}),
            'ret_value': ('django.db.models.fields.IntegerField', [], {})
        },
        'openproximity.remotebluetoothdevicefilessuccess': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDeviceFilesSuccess', '_ormbases': ['openproximity.RemoteBluetoothDeviceFileTry']},
            'remotebluetoothdevicefiletry_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceFileTry']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicefiletry': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDeviceFileTry', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['openproximity.MarketingCampaign']"}),
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicefoundrecord': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDeviceFoundRecord', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            '_RemoteBluetoothDeviceFoundRecord__rssi': ('django.db.models.fields.CommaSeparatedIntegerField', [], {'max_length': '200'}),
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicenosdp': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDeviceNoSDP', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicepairing': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDevicePairing', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'exception': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'msg': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'}),
            'state': ('django.db.models.fields.IntegerField', [], {})
        },
        'openproximity.remotebluetoothdevicependingaction': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDevicePendingAction', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'action': ('django.db.models.fields.IntegerField', [], {}),
            'end': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'length': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicerecord': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDeviceRecord', '_ormbases': ['openproximity.DeviceRecord']},
            'devicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.DeviceRecord']", 'unique': 'True', 'primary_key': 'True'}),
            'remote': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['openproximity.RemoteDevice']"})
        },
        'openproximity.remotebluetoothdevicesdp': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDeviceSDP', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'channel': ('django.db.models.fields.IntegerField', [], {}),
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotebluetoothdevicesdptimeout': {
            'Meta': {'ordering': "['time']", 'object_name': 'RemoteBluetoothDeviceSDPTimeout', '_ormbases': ['openproximity.RemoteBluetoothDeviceRecord']},
            'remotebluetoothdevicerecord_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.RemoteBluetoothDeviceRecord']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.remotedevice': {
            'Meta': {'object_name': 'RemoteDevice'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '17'}),
            'devclass': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_seen': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'openproximity.remotescannerbluetoothdongle': {
            'Meta': {'object_name': 'RemoteScannerBluetoothDongle', '_ormbases': ['openproximity.ScannerBluetoothDongle']},
            'local_dongle': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'remote_dongles'", 'to': "orm['openproximity.ScannerBluetoothDongle']"}),
            'scannerbluetoothdongle_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.ScannerBluetoothDongle']", 'unique': 'True', 'primary_key': 'True'})
        },
        'openproximity.scannerbluetoothdongle': {
            'Meta': {'object_name': 'ScannerBluetoothDongle', '_ormbases': ['openproximity.BluetoothDongle']},
            'bluetoothdongle_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.BluetoothDongle']", 'unique': 'True', 'primary_key': 'True'}),
            'priority': ('django.db.models.fields.IntegerField', [], {})
        },
        'openproximity.setting': {
            'Meta': {'object_name': 'Setting'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'value': ('net.aircable.fields.PickledField', [], {'max_length': '200'})
        },
        'openproximity.uploaderbluetoothdongle': {
            'Meta': {'object_name': 'UploaderBluetoothDongle', '_ormbases': ['openproximity.BluetoothDongle']},
            'bluetoothdongle_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['openproximity.BluetoothDongle']", 'unique': 'True', 'primary_key': 'True'}),
            'max_conn': ('django.db.models.fields.IntegerField', [], {'default': '7'})
        },
        'openproximity.userprofile': {
            'Meta': {'object_name': 'UserProfile'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'timezone': ('timezones.fields.TimeZoneField', [], {'default': "'America/Los_Angeles'", 'max_length': '100'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True'})
        }
    }

    complete_apps = ['openproximity']
